@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2006 and 2009 Chris Vine
+/* Copyright (C) 2001 to 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -31,14 +31,13 @@ COPYING distributed with the source files.
 
 int AddressBook::is_address_list = 0;
 
-
+namespace {
 namespace ModelColumns {
   enum {name, number, cols_num};
-}
+} // namespace ModelColumns
+} // anonymous namespace
 
 
-namespace { // callbacks for internal use only
-
 void AddressBookCB::addr_book_button_clicked(GtkWidget* widget_p, void* data) {
   AddressBook* instance_p = static_cast<AddressBook*>(data);
   
@@ -71,8 +70,6 @@ void AddressBookCB::addr_book_drag_n_drop(GtkTreeModel*, GtkTreePath*, void* dat
   static_cast<AddressBook*>(data)->save_list();
 }
 
-} // anonymous namespace
-
 
 AddressBook::AddressBook(const int size, GtkWindow* parent_p):
                              WinBase(gettext("efax-gtk: Address book"),
@@ -255,7 +252,7 @@ AddressBook::~AddressBook(void) {
 }
 
 void AddressBook::on_delete_event(void) {
-  gtk_widget_hide_all(GTK_WIDGET(get_win()));
+  gtk_widget_hide(GTK_WIDGET(get_win()));
   result = "";
   close();
 }
@@ -530,8 +527,6 @@ void AddressBook::move_down_impl(void) {
   else beep();
 }
 
-namespace { // callbacks for internal use only
-
 void AddressDialogCB::addr_dialog_selected(GtkWidget* widget_p, void* data) {
   AddressDialog* instance_p = static_cast<AddressDialog*>(data);
 
@@ -545,7 +540,6 @@ void AddressDialogCB::addr_dialog_selected(GtkWidget* widget_p, void* data) {
   }
 }
 
-
 gboolean AddressDialogCB::addr_dialog_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
   AddressDialog* instance_p = static_cast<AddressDialog*>(data);
   int keycode = event_p->keyval;
@@ -555,15 +549,20 @@ gboolean AddressDialogCB::addr_dialog_key_press_event(GtkWidget*, GdkEventKey* e
   bool cancel_focus = GTK_WIDGET_HAS_FOCUS(instance_p->cancel_button_p);
 #endif
 
+#if GTK_CHECK_VERSION(2,91,0)
+  if (keycode == GDK_KEY_Return && !cancel_focus) {
+    if (instance_p->selected_impl()) instance_p->close();
+    return true;  // stop processing here
+  }
+#else
   if (keycode == GDK_Return && !cancel_focus) {
     if (instance_p->selected_impl()) instance_p->close();
     return true;  // stop processing here
   }
+#endif
   return false;   // continue processing key events
 }
 
-} // anonymous namespace
-
 AddressDialog::AddressDialog(const int standard_size, GtkWindow* parent_p):
                              WinBase(gettext("efax-gtk: Add address"),
 				     prog_config.window_icon_h,
