@@ -32,26 +32,22 @@ COPYING distributed with the source files.
 #endif
 
 
-namespace { // callbacks for internal use only
-
-void FileReadSelectDialogCB::file_selected(GtkWidget* widget_p, void* data) {
+void FileReadSelectDialogCB::frs_dialog_selected(GtkWidget* widget_p, void* data) {
   FileReadSelectDialog* instance_p = static_cast<FileReadSelectDialog*>(data);
-  gtk_widget_hide_all(GTK_WIDGET(instance_p->get_win()));
+  gtk_widget_hide(GTK_WIDGET(instance_p->get_win()));
   if (widget_p == instance_p->ok_button_p) instance_p->set_result();
   else if (widget_p == instance_p->cancel_button_p) instance_p->result.clear();
   else {
     instance_p->result.clear();
-    write_error("Callback error in FileReadSelectDialogCB::file_selected()\n");
+    write_error("Callback error in FileReadSelectDialogCB::frs_dialog_selected()\n");
   }
   instance_p->close();
 }
 
-void FileReadSelectDialogCB::file_view_file(GtkWidget*, void* data) {
+void FileReadSelectDialogCB::frs_dialog_view_file(GtkWidget*, void* data) {
   static_cast<FileReadSelectDialog*>(data)->view_file_impl();
 }
 
-} // anonymous namespace
-
 FileReadSelectDialog::FileReadSelectDialog(int size, bool multi_files, GtkWindow* parent_p):
             WinBase(gettext("efax-gtk: File to fax"),
 		    prog_config.window_icon_h,
@@ -97,11 +93,11 @@ FileReadSelectDialog::FileReadSelectDialog(int size, bool multi_files, GtkWindow
   g_list_free(button_list);
 
   g_signal_connect(G_OBJECT(ok_button_p), "clicked",
-		   G_CALLBACK(FileReadSelectDialogCB::file_selected), this);
+		   G_CALLBACK(FileReadSelectDialogCB::frs_dialog_selected), this);
   g_signal_connect(G_OBJECT(cancel_button_p), "clicked",
-		   G_CALLBACK(FileReadSelectDialogCB::file_selected), this);
+		   G_CALLBACK(FileReadSelectDialogCB::frs_dialog_selected), this);
   g_signal_connect(G_OBJECT(view_button_p), "clicked",
-		   G_CALLBACK(FileReadSelectDialogCB::file_view_file), this);
+		   G_CALLBACK(FileReadSelectDialogCB::frs_dialog_view_file), this);
 
   gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
   gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
@@ -110,7 +106,7 @@ FileReadSelectDialog::FileReadSelectDialog(int size, bool multi_files, GtkWindow
 }
 
 void FileReadSelectDialog::on_delete_event(void) {
-  gtk_widget_hide_all(GTK_WIDGET(get_win()));
+  gtk_widget_hide(GTK_WIDGET(get_win()));
   result.clear();
   close();
 }
@@ -261,31 +257,38 @@ void FileReadSelectDialog::set_result(void) {
 }
 
 
-namespace { // callbacks for internal use only
-
-void GplDialogCB::gpl_selected(GtkWidget* widget_p, void* data) {
+void GplDialogCB::gpl_dialog_selected(GtkWidget* widget_p, void* data) {
   GplDialog* instance_p = static_cast<GplDialog*>(data);
-  gtk_widget_hide_all(GTK_WIDGET(instance_p->get_win()));
+  gtk_widget_hide(GTK_WIDGET(instance_p->get_win()));
   if (widget_p == instance_p->accept_button_p) instance_p->result = GplDialog::accepted;
   else if (widget_p == instance_p->reject_button_p) instance_p->result = GplDialog::rejected;
-  else write_error("Callback error in GplDialogCB::gpl_selected()\n");
+  else write_error("Callback error in GplDialogCB::gpl_dialog_selected()\n");
   instance_p->close();
 }
 
-gboolean GplDialogCB::gpl_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
+gboolean GplDialogCB::gpl_dialog_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
   GplDialog* instance_p = static_cast<GplDialog*>(data);
   int keycode = event_p->keyval;
   
+#if GTK_CHECK_VERSION(2,91,0)
+  if (keycode == GDK_KEY_Escape) instance_p->on_delete_event();
+  else if (keycode == GDK_KEY_Home || keycode == GDK_KEY_End
+	   || keycode == GDK_KEY_Up || keycode == GDK_KEY_Down
+	   || keycode == GDK_KEY_Page_Up || keycode == GDK_KEY_Page_Down) {
+    gtk_widget_event(GTK_WIDGET(instance_p->text_view_p), (GdkEvent*)event_p);
+  }
+#else
   if (keycode == GDK_Escape) instance_p->on_delete_event();
   else if (keycode == GDK_Home || keycode == GDK_End
 	   || keycode == GDK_Up || keycode == GDK_Down
 	   || keycode == GDK_Page_Up || keycode == GDK_Page_Down) {
     gtk_widget_event(GTK_WIDGET(instance_p->text_view_p), (GdkEvent*)event_p);
   }
+#endif
   return true;    // stop processing here
 }
 
-void GplDialogCB::gpl_style_set(GtkWidget*, GtkStyle*, void* data) {
+void GplDialogCB::gpl_dialog_style_set(GtkWidget*, GtkStyle*, void* data) {
 
   GplDialog* instance_p = static_cast<GplDialog*>(data);
 
@@ -316,8 +319,6 @@ void GplDialogCB::gpl_style_set(GtkWidget*, GtkStyle*, void* data) {
   gtk_widget_set_size_request(instance_p->reject_button_p, width, height);
 }
 
-} // anonymous namespace
-
 GplDialog::GplDialog(int size): WinBase(gettext("efax-gtk: Conditions, Notices and Disclaimers"),
 				        prog_config.window_icon_h,
 					true),
@@ -373,13 +374,13 @@ GplDialog::GplDialog(int size): WinBase(gettext("efax-gtk: Conditions, Notices a
 		   0, standard_size/3);
 
   g_signal_connect(G_OBJECT(get_win()), "style_set",
-		   G_CALLBACK(GplDialogCB::gpl_style_set), this);
+		   G_CALLBACK(GplDialogCB::gpl_dialog_style_set), this);
   g_signal_connect(G_OBJECT(accept_button_p), "clicked",
-		   G_CALLBACK(GplDialogCB::gpl_selected), this);
+		   G_CALLBACK(GplDialogCB::gpl_dialog_selected), this);
   g_signal_connect(G_OBJECT(reject_button_p), "clicked",
-		   G_CALLBACK(GplDialogCB::gpl_selected), this);
+		   G_CALLBACK(GplDialogCB::gpl_dialog_selected), this);
   g_signal_connect(G_OBJECT(get_win()), "key_press_event",
-		   G_CALLBACK(GplDialogCB::gpl_key_press_event), this);
+		   G_CALLBACK(GplDialogCB::gpl_dialog_key_press_event), this);
 
   gtk_container_add(GTK_CONTAINER(get_win()), GTK_WIDGET(table_p));
   
@@ -408,7 +409,7 @@ int GplDialog::get_exec_val(void) const {
 }
 
 void GplDialog::on_delete_event(void) {
-  gtk_widget_hide_all(GTK_WIDGET(get_win()));
+  gtk_widget_hide(GTK_WIDGET(get_win()));
   result = rejected;
   close();
 }
@@ -443,14 +444,10 @@ void GplDialog::get_longest_button_text(void) {
 }
 
 
-namespace { // callbacks for internal use only
-
-void InfoDialogCB::info_selected(GtkDialog*, int, void* data) {
+void InfoDialogCB::info_dialog_selected(GtkDialog*, int, void* data) {
   static_cast<InfoDialog*>(data)->close();
 }
 
-} // anonymous namespace
-
 InfoDialog::InfoDialog(const char* text, const char* caption,
                        GtkMessageType message_type, GtkWindow* parent_p, bool modal):
                        WinBase(caption, prog_config.window_icon_h,
@@ -471,7 +468,7 @@ InfoDialog::InfoDialog(const char* text, const char* caption,
   gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
 
   g_signal_connect(G_OBJECT(get_win()), "response",
-		   G_CALLBACK(InfoDialogCB::info_selected), this);
+		   G_CALLBACK(InfoDialogCB::info_dialog_selected), this);
 
   gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
   gtk_window_set_resizable(get_win(), false);
@@ -486,9 +483,7 @@ void InfoDialog::on_delete_event(void) {
   return;
 }
 
-namespace { // callbacks for internal use only
-
-void PromptDialogCB::prompt_selected(GtkDialog*, int id, void* data) {
+void PromptDialogCB::prompt_dialog_selected(GtkDialog*, int id, void* data) {
   PromptDialog* instance_p = static_cast<PromptDialog*>(data);
   if (id == GTK_RESPONSE_ACCEPT) {
     instance_p->result = true;
@@ -502,14 +497,16 @@ void PromptDialogCB::prompt_selected(GtkDialog*, int id, void* data) {
   }
 }
 
-} // anonymous namespace
-
 PromptDialog::PromptDialog(const char* text, const char* caption,
 			   int standard_size, GtkWindow* parent_p, bool modal):
                            WinBase(caption, prog_config.window_icon_h,
 				   modal, parent_p,
 				   GTK_WINDOW(gtk_dialog_new_with_buttons(0, 0,
+#if GTK_CHECK_VERSION(2,91,0)
+						  GtkDialogFlags(0),
+#else
 						  GTK_DIALOG_NO_SEPARATOR,
+#endif
 						  GTK_STOCK_CANCEL,
 						  GTK_RESPONSE_REJECT,
 						  GTK_STOCK_OK,
@@ -533,7 +530,7 @@ PromptDialog::PromptDialog(const char* text, const char* caption,
   gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
 
   g_signal_connect(G_OBJECT(get_win()), "response",
-		   G_CALLBACK(PromptDialogCB::prompt_selected), this);
+		   G_CALLBACK(PromptDialogCB::prompt_dialog_selected), this);
 
   gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
   gtk_window_set_resizable(get_win(), false);
@@ -551,14 +548,10 @@ void PromptDialog::on_delete_event(void) {
   close();
 }
 
-namespace { // callbacks for internal use only
-
-void AboutEfaxGtkDialogCB::about_efax_gtk_selected(GtkDialog*, int, void* data) {
+void AboutEfaxGtkDialogCB::about_efax_gtk_dialog_selected(GtkDialog*, int, void* data) {
   static_cast<AboutEfaxGtkDialog*>(data)->close();
 }
 
-} // anonymous namespace
-
 AboutEfaxGtkDialog::AboutEfaxGtkDialog(GtkWindow* parent_p, bool modal):
                                        WinBase(gettext("About efax-gtk"), prog_config.window_icon_h,
 					       modal, parent_p,
@@ -567,7 +560,7 @@ AboutEfaxGtkDialog::AboutEfaxGtkDialog(GtkWindow* parent_p, bool modal):
   gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
 
   g_signal_connect(G_OBJECT(get_win()), "response",
-		   G_CALLBACK(AboutEfaxGtkDialogCB::about_efax_gtk_selected), this);
+		   G_CALLBACK(AboutEfaxGtkDialogCB::about_efax_gtk_dialog_selected), this);
 
   gtk_window_set_resizable(get_win(), false);
 
@@ -595,14 +588,10 @@ void AboutEfaxGtkDialog::on_delete_event(void) {
   return;
 }
 
-namespace { // callbacks for internal use only
-
-void AboutEfaxDialogCB::about_efax_selected(GtkDialog*, int, void* data) {
+void AboutEfaxDialogCB::about_efax_dialog_selected(GtkDialog*, int, void* data) {
   static_cast<AboutEfaxDialog*>(data)->close();
 }
 
-} // anonymous namespace
-
 AboutEfaxDialog::AboutEfaxDialog(GtkWindow* parent_p, bool modal):
                                  WinBase(gettext("About efax"), prog_config.window_icon_h,
 					 modal, parent_p,
@@ -619,7 +608,7 @@ AboutEfaxDialog::AboutEfaxDialog(GtkWindow* parent_p, bool modal):
   gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
 
   g_signal_connect(G_OBJECT(get_win()), "response",
-		   G_CALLBACK(AboutEfaxDialogCB::about_efax_selected), this);
+		   G_CALLBACK(AboutEfaxDialogCB::about_efax_dialog_selected), this);
 
   gtk_window_set_resizable(get_win(), false);
 
