@@ -22,15 +22,12 @@ COPYING distributed with the source files.
 #include <c++-gtk-utils/emitter.h>
 
 
-namespace { // we put the functions in anonymous namespace in dialogs.cpp
-            // so they are not exported at link time
 namespace FileReadSelectDialogCB {
   extern "C" {
-    void file_selected(GtkWidget*, void*);
-    void file_view_file(GtkWidget*, void*);
+    void frs_dialog_selected(GtkWidget*, void*);
+    void frs_dialog_view_file(GtkWidget*, void*);
   }
 }
-}   // anonymous namespace
 
 class FileReadSelectDialog: public WinBase {
   int standard_size;
@@ -47,24 +44,21 @@ class FileReadSelectDialog: public WinBase {
 protected:
   virtual void on_delete_event(void);
 public:
-  friend void FileReadSelectDialogCB::file_selected(GtkWidget*, void*);
-  friend void FileReadSelectDialogCB::file_view_file(GtkWidget*, void*);
+  friend void FileReadSelectDialogCB::frs_dialog_selected(GtkWidget*, void*);
+  friend void FileReadSelectDialogCB::frs_dialog_view_file(GtkWidget*, void*);
 
   // get_result() returns the filenames in UTF-8 (not filesystem) encoding
   std::vector<std::string> get_result(void) const {return result;}
   FileReadSelectDialog(int standard_size, bool multi_files, GtkWindow* parent_p);
 };
 
-namespace { // we put the functions in anonymous namespace in dialogs.cpp
-            // so they are not exported at link time
 namespace GplDialogCB {
   extern "C" {
-    void gpl_selected(GtkWidget*, void*);
-    gboolean gpl_key_press_event(GtkWidget*, GdkEventKey*, void*);
-    void gpl_style_set(GtkWidget*, GtkStyle*, void*);
+    void gpl_dialog_selected(GtkWidget*, void*);
+    gboolean gpl_dialog_key_press_event(GtkWidget*, GdkEventKey*, void*);
+    void gpl_dialog_style_set(GtkWidget*, GtkStyle*, void*);
   }
 }
-}   // anonymous namespace
 
 class GplDialog: public WinBase {
 public:
@@ -81,36 +75,30 @@ protected:
   virtual int get_exec_val(void) const;
   virtual void on_delete_event(void);
 public:
-  friend void GplDialogCB::gpl_selected(GtkWidget*, void*);
-  friend gboolean GplDialogCB::gpl_key_press_event(GtkWidget*, GdkEventKey*, void*);
-  friend void GplDialogCB::gpl_style_set(GtkWidget*, GtkStyle*, void*);
+  friend void GplDialogCB::gpl_dialog_selected(GtkWidget*, void*);
+  friend gboolean GplDialogCB::gpl_dialog_key_press_event(GtkWidget*, GdkEventKey*, void*);
+  friend void GplDialogCB::gpl_dialog_style_set(GtkWidget*, GtkStyle*, void*);
 
   GplDialog(int standard_size);
 };
 
-namespace { // we put the function in anonymous namespace in dialogs.cpp
-            // so it is not exported at link time
 namespace InfoDialogCB {
-  extern "C" void info_selected(GtkDialog*, int, void*);
+  extern "C" void info_dialog_selected(GtkDialog*, int, void*);
 }
-}   // anonymous namespace
 
 class InfoDialog: public WinBase {
 protected:
   virtual void on_delete_event(void);
 public:
-  friend void InfoDialogCB::info_selected(GtkDialog*, int, void*);
+  friend void InfoDialogCB::info_dialog_selected(GtkDialog*, int, void*);
 
   InfoDialog(const char* text, const char* caption,
 	     GtkMessageType message_type, GtkWindow* parent_p, bool modal = true);
 };
 
-namespace { // we put the function in anonymous namespace in dialogs.cpp
-            // so it is not exported at link time
 namespace PromptDialogCB {
-  extern "C" void prompt_selected(GtkDialog*, int, void*);
+  extern "C" void prompt_dialog_selected(GtkDialog*, int, void*);
 }
-}   // anonymous namespace
 
 class PromptDialog: public WinBase {
   bool result;
@@ -118,7 +106,7 @@ protected:
   virtual int get_exec_val(void) const;
   virtual void on_delete_event(void);
 public:
-  friend void PromptDialogCB::prompt_selected(GtkDialog*, int, void*);
+  friend void PromptDialogCB::prompt_dialog_selected(GtkDialog*, int, void*);
 
   Emitter accepted;
   Emitter rejected;
@@ -127,34 +115,28 @@ public:
 	       int standard_size, GtkWindow* parent_p, bool modal = true);
 };
 
-namespace { // we put the function in anonymous namespace in dialogs.cpp
-            // so it is not exported at link time
 namespace AboutEfaxGtkDialogCB {
-  extern "C" void about_efax_gtk_selected(GtkDialog*, int, void*);
+  extern "C" void about_efax_gtk_dialog_selected(GtkDialog*, int, void*);
 }
-}   // anonymous namespace
 
 class AboutEfaxGtkDialog: public WinBase {
 protected:
   virtual void on_delete_event(void);
 public:
-  friend void AboutEfaxGtkDialogCB::about_efax_gtk_selected(GtkDialog*, int, void*);
+  friend void AboutEfaxGtkDialogCB::about_efax_gtk_dialog_selected(GtkDialog*, int, void*);
 
   AboutEfaxGtkDialog(GtkWindow* parent_p = 0, bool modal = false);
 };
 
-namespace { // we put the function in anonymous namespace in dialogs.cpp
-            // so it is not exported at link time
 namespace AboutEfaxDialogCB {
-  extern "C" void about_efax_selected(GtkDialog*, int, void*);
+  extern "C" void about_efax_dialog_selected(GtkDialog*, int, void*);
 }
-}   // anonymous namespace
 
 class AboutEfaxDialog: public WinBase {
 protected:
   virtual void on_delete_event(void);
 public:
-  friend void AboutEfaxDialogCB::about_efax_selected(GtkDialog*, int, void*);
+  friend void AboutEfaxDialogCB::about_efax_dialog_selected(GtkDialog*, int, void*);
 
   AboutEfaxDialog(GtkWindow* parent_p = 0, bool modal = false);
 };
