@@ -50,6 +50,13 @@ COPYING distributed with the source files.
 #include <libintl.h>
 #endif
 
+// there can be certain circumstances where PIPE_BUF is not
+// defined in <limits.h>.  If so, just define the minimum
+// required by POSIX
+#ifndef PIPE_BUF
+#define PIPE_BUF 512
+#endif
+
 // the headers on one of my machines does not properly include the declaration
 // of getpgid() for some reason, so declare it here explicitly
 extern "C" pid_t getpgid(pid_t);
@@ -494,8 +501,7 @@ void EfaxController::ps_file_to_tiff_files(const std::string& filename) {
   } while (ret == -1 && errno == EINTR);
 
   if (ret == -1 || !WIFEXITED(status)) {
-    write_error(gettext("Not valid postscript file\n"));
-    throw bool();
+    throw std::string(gettext("Not valid postscript/PDF file\n"));
   }
 
   // now enter the names of the created files in efax_parms_vec
@@ -532,8 +538,7 @@ void EfaxController::ps_file_to_tiff_files(const std::string& filename) {
   }
     
   if (!valid_file) {
-    write_error(gettext("Not valid postscript file\n"));
-    throw bool();
+    throw std::string(gettext("Not valid postscript/PDF file\n"));
   }
 }
 
