@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2007 and 2009 Chris Vine
+/* Copyright (C) 2001 to 2007, 2009 and 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -47,12 +47,17 @@ COPYING distributed with the source files.
 #include <libintl.h>
 #endif
 
+// there can be certain circumstances where PIPE_BUF is not
+// defined in <limits.h>.  If so, just define the minimum
+// required by POSIX
+#ifndef PIPE_BUF
+#define PIPE_BUF 512
+#endif
+
 int FaxListDialog::is_fax_received_list = 0;
 int FaxListDialog::is_fax_sent_list = 0;
 
-namespace { // callback for internal use only
-
-void FaxListDialogCB::fax_list_button_clicked(GtkWidget* widget_p, void* data) {
+void FaxListDialogCB::fax_list_dialog_button_clicked(GtkWidget* widget_p, void* data) {
   FaxListDialog* instance_p = static_cast<FaxListDialog*>(data);
   
   if (widget_p == instance_p->close_button_p) {
@@ -87,13 +92,11 @@ void FaxListDialogCB::fax_list_button_clicked(GtkWidget* widget_p, void* data) {
     MonoTiffPrintManager::page_setup(instance_p->get_win());
   }
   else {
-    write_error("Callback error in FaxListDialogCB::fax_list_book_button_clicked()\n");
+    write_error("Callback error in FaxListDialogCB::fax_list_dialog_button_clicked()\n");
     instance_p->close();
   }
 }
 
-}  // anonymous namespace
-
 FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
 			      WinBase(0, prog_config.window_icon_h, false),
                               mode(mode_), standard_size(standard_size_),
@@ -188,7 +191,7 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
 			gettext("Print page setup"));
   set_page_setup_button();
   g_signal_connect(G_OBJECT(page_setup_button_p), "clicked",
-		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
+		   G_CALLBACK(FaxListDialogCB::fax_list_dialog_button_clicked), this);
 
   image_p = gtk_image_new_from_stock(GTK_STOCK_PRINT, efax_gtk_button_size);
   gtk_container_add(GTK_CONTAINER(print_button_p), image_p);
@@ -279,21 +282,21 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
 		   standard_size/3, standard_size/3);
 
   g_signal_connect(G_OBJECT(close_button_p), "clicked",
-		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
+		   G_CALLBACK(FaxListDialogCB::fax_list_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(print_button_p), "clicked",
-		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
+		   G_CALLBACK(FaxListDialogCB::fax_list_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(view_button_p), "clicked",
-		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
+		   G_CALLBACK(FaxListDialogCB::fax_list_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(describe_button_p), "clicked",
-		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
+		   G_CALLBACK(FaxListDialogCB::fax_list_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(delete_fax_button_p), "clicked",
-		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
+		   G_CALLBACK(FaxListDialogCB::fax_list_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(empty_trash_button_p), "clicked",
-		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
+		   G_CALLBACK(FaxListDialogCB::fax_list_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(add_folder_button_p), "clicked",
-		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
+		   G_CALLBACK(FaxListDialogCB::fax_list_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(delete_folder_button_p), "clicked",
-		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
+		   G_CALLBACK(FaxListDialogCB::fax_list_dialog_button_clicked), this);
 
   GtkWidget* toolbar_box_p = gtk_hbox_new(false, 0);
   gtk_box_pack_start(GTK_BOX(toolbar_box_p), GTK_WIDGET(toolbar_p), true, true, 0);
@@ -311,7 +314,7 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
     gtk_widget_set_sensitive(reset_button_p, true);
     gtk_button_set_relief(GTK_BUTTON(reset_button_p), GTK_RELIEF_NORMAL);
     g_signal_connect(G_OBJECT(reset_button_p), "clicked",
-		     G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
+		     G_CALLBACK(FaxListDialogCB::fax_list_dialog_button_clicked), this);
 
     // now provide a display for the new received fax count
     new_fax_frame_p = gtk_frame_new(0);
@@ -1289,7 +1292,6 @@ void FaxListDialog::delete_parms(std::pair<const char*, char* const*> parms_pair
   delete[] parms_pair.second;
 }
 
-namespace { // callbacks for internal use only
 
 void EntryDialogCB::entry_dialog_selected(GtkWidget* widget_p, void* data) {
   EntryDialog* instance_p = static_cast<EntryDialog*>(data);
@@ -1304,7 +1306,6 @@ void EntryDialogCB::entry_dialog_selected(GtkWidget* widget_p, void* data) {
   }
 }
 
-
 gboolean EntryDialogCB::entry_dialog_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
   EntryDialog* instance_p = static_cast<EntryDialog*>(data);
   int keycode = event_p->keyval;
@@ -1313,15 +1314,22 @@ gboolean EntryDialogCB::entry_dialog_key_press_event(GtkWidget*, GdkEventKey* ev
 #else
   bool cancel_focus = GTK_WIDGET_HAS_FOCUS(instance_p->cancel_button_p);
 #endif
+
+#if GTK_CHECK_VERSION(2,91,0)
+  if (keycode == GDK_KEY_Return && !cancel_focus) {
+    if (instance_p->selected_impl()) instance_p->close();
+    return true;  // stop processing here
+  }
+#else
   if (keycode == GDK_Return && !cancel_focus) {
     if (instance_p->selected_impl()) instance_p->close();
     return true;  // stop processing here
   }
+#endif
+
   return false;   // continue processing key events
 }
 
-} // anonymous namespace
-
 EntryDialog::EntryDialog(const int standard_size, const char* entry_text,
 			 const char* caption, const char* label_text,
 			 GtkWindow* parent_p):
