@@ -21,6 +21,9 @@ COPYING distributed with the source files.
 #include "fax_list_manager.h"
 #include "fax_list_manager_icons.h"
 #include "utils/tree_path_handle.h"
+#if GTK_CHECK_VERSION(2,91,0)
+#include "utils/cairo_handle.h"
+#endif
 
 #include <c++-gtk-utils/timeout.h>
 #include <c++-gtk-utils/convert.h>
@@ -43,6 +46,7 @@ bool FaxListManager::fax_received_list_main_iteration = false;
 bool FaxListManager::fax_sent_list_main_iteration = false;
 GtkSortType FaxListManager::list_sort_type[FaxListEnum::sent + 1] = {GTK_SORT_ASCENDING, GTK_SORT_ASCENDING};
 
+namespace {
 namespace FolderListModelColumns {
   enum {icon, name, root_only, cols_num};
 }
@@ -50,6 +54,7 @@ namespace FolderListModelColumns {
 namespace FaxListModelColumns {
   enum {fax_date, fax_description, name, cols_num};
 }
+} // anonymous namespace
 
 std::pair<bool, std::string> FolderNameValidator::validate(const std::string& folder_name) {
 
@@ -74,275 +79,366 @@ std::pair<bool, std::string> FolderNameValidator::validate(const std::string& fo
   return return_val;
 }
 
-namespace { // callbacks for internal use only
+class FaxListManager::CB {
+public:
+  static void folder_tree_view_drag_begin(GdkDragContext* context_p,
+					  void* data) {
 
-void FaxListManagerCB::folder_tree_view_drag_begin(GtkWidget*, GdkDragContext* context_p,
-						   void* data) {
+    FaxListManager* instance_p = static_cast<FaxListManager*>(data);
+    instance_p->drag_is_fax = false;
 
-  FaxListManager* instance_p = static_cast<FaxListManager*>(data);
-  instance_p->drag_is_fax = false;
+    GtkTreeModel* model_p;
+    GtkTreeSelection* selection_p = gtk_tree_view_get_selection(instance_p->folder_tree_view_p);
+    if (gtk_tree_selection_get_selected(selection_p, &model_p,
+					&instance_p->folder_drag_source_row_iter)) {
 
-  GtkTreeModel* model_p;
-  GtkTreeSelection* selection_p = gtk_tree_view_get_selection(instance_p->folder_tree_view_p);
-  if (gtk_tree_selection_get_selected(selection_p, &model_p,
-				      &instance_p->folder_drag_source_row_iter)) {
+      GtkTreeViewColumn* column_p = gtk_tree_view_get_column(instance_p->folder_tree_view_p, 0);
+      gint x_off, y_off, cell_width, cell_height;
+      GdkRectangle cell_area;
+      gtk_tree_view_column_cell_get_size(column_p, &cell_area, &x_off, &y_off, &cell_width, &cell_height);
 
-    GtkTreeViewColumn* column_p = gtk_tree_view_get_column(instance_p->folder_tree_view_p, 0);
-    gint x_off, y_off, cell_width, cell_height;
-    GdkRectangle cell_area;
-    gtk_tree_view_column_cell_get_size(column_p, &cell_area, &x_off, &y_off, &cell_width, &cell_height);
-
-    if (cell_height < 0) cell_height = 0;
+      if (cell_height < 0) cell_height = 0;
     
-    TreePathScopedHandle path_h(gtk_tree_model_get_path(model_p,
-							&instance_p->folder_drag_source_row_iter));
+      TreePathScopedHandle path_h(gtk_tree_model_get_path(model_p,
+							  &instance_p->folder_drag_source_row_iter));
     
-    // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
-    GobjHandle<GdkPixmap> pixmap_h(gtk_tree_view_create_row_drag_icon(instance_p->folder_tree_view_p, path_h));
-    GdkColormap* colormap_p = gtk_widget_get_default_colormap();
-
-    gtk_drag_set_icon_pixmap(context_p, colormap_p, pixmap_h, 0, 5, cell_height/2);
+#if GTK_CHECK_VERSION(2,91,0)
+      CairoSurfaceScopedHandle icon_h(gtk_tree_view_create_row_drag_icon(instance_p->folder_tree_view_p,
+									 path_h));
+      cairo_surface_set_device_offset(icon_h, -5, -cell_height/2);
+      gtk_drag_set_icon_surface(context_p, icon_h);
+#else
+      // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
+      GobjHandle<GdkPixmap> icon_h(gtk_tree_view_create_row_drag_icon(instance_p->folder_tree_view_p,
+								      path_h));
+      GdkColormap* colormap_p = gtk_widget_get_default_colormap();
+      gtk_drag_set_icon_pixmap(context_p, colormap_p, icon_h, 0, 5, cell_height/2);
+#endif
+    }
   }
-}
 
-void FaxListManagerCB::fax_tree_view_drag_begin(GtkWidget*, GdkDragContext* context_p,
-						void* data) {
+  static void fax_tree_view_drag_begin(GdkDragContext* context_p,
+				       void* data) {
 
-  FaxListManager* instance_p = static_cast<FaxListManager*>(data);
-  instance_p->drag_is_fax = true;
+    FaxListManager* instance_p = static_cast<FaxListManager*>(data);
+    instance_p->drag_is_fax = true;
 
-  SelectedRowsHandle rows_handle(instance_p->fax_tree_view_p);
-  rows_handle.get_ref_list(gtk_tree_view_get_model(instance_p->fax_tree_view_p),
-			   instance_p->fax_drag_source_row_refs);
+    SelectedRowsHandle rows_handle(instance_p->fax_tree_view_p);
+    rows_handle.get_ref_list(gtk_tree_view_get_model(instance_p->fax_tree_view_p),
+			     instance_p->fax_drag_source_row_refs);
 
-  // if we are dragging only one row, provide a row icon, but otherwise
-  // use the generic GTK+ drag icon
-  if (rows_handle.size() == 1 &&
-      gtk_tree_row_reference_valid(instance_p->fax_drag_source_row_refs.front())) {
+    // if we are dragging only one row, provide a row icon, but otherwise
+    // use the generic GTK+ drag icon
+    if (rows_handle.size() == 1 &&
+	gtk_tree_row_reference_valid(instance_p->fax_drag_source_row_refs.front())) {
 
-    GtkTreeViewColumn* column_p = gtk_tree_view_get_column(instance_p->fax_tree_view_p, 0);
-    gint x_off, y_off, cell_width, cell_height;
-    GdkRectangle cell_area;
-    gtk_tree_view_column_cell_get_size(column_p, &cell_area, &x_off, &y_off, &cell_width, &cell_height);
+      GtkTreeViewColumn* column_p = gtk_tree_view_get_column(instance_p->fax_tree_view_p, 0);
+      gint x_off, y_off, cell_width, cell_height;
+      GdkRectangle cell_area;
+      gtk_tree_view_column_cell_get_size(column_p, &cell_area, &x_off, &y_off, &cell_width, &cell_height);
 
-    gint x_pos;
-    gtk_widget_get_pointer(GTK_WIDGET(instance_p->fax_tree_view_p), &x_pos, 0);
+      gint x_pos;
+      gtk_widget_get_pointer(GTK_WIDGET(instance_p->fax_tree_view_p), &x_pos, 0);
 
-    if (cell_height < 0) cell_height = 0;
-    if (x_pos < 0) x_pos = 0;
+      if (cell_height < 0) cell_height = 0;
+      if (x_pos < 0) x_pos = 0;
 
-    // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
-    GobjHandle<GdkPixmap> pixmap_h(gtk_tree_view_create_row_drag_icon(instance_p->fax_tree_view_p,
+#if GTK_CHECK_VERSION(2,91,0)
+      CairoSurfaceScopedHandle icon_h(gtk_tree_view_create_row_drag_icon(instance_p->fax_tree_view_p,
+									 rows_handle.front()));
+      cairo_surface_set_device_offset(icon_h, -x_pos, -cell_height/2);
+      gtk_drag_set_icon_surface(context_p, icon_h);
+#else
+      // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
+      GobjHandle<GdkPixmap> icon_h(gtk_tree_view_create_row_drag_icon(instance_p->fax_tree_view_p,
 								      rows_handle.front()));
-    GdkColormap* colormap_p = gtk_widget_get_default_colormap();
+      GdkColormap* colormap_p = gtk_widget_get_default_colormap();
 
-    gtk_drag_set_icon_pixmap(context_p, colormap_p, pixmap_h, 0, x_pos, cell_height/2);
-  }
-}
-
-gboolean FaxListManagerCB::drag_motion(GtkWidget*, GdkDragContext* context_p,
-				       gint x, gint y, guint time, void* data) {
-
-  FaxListManager* instance_p = static_cast<FaxListManager*>(data);
-
-  bool return_val;
-  GtkTreePath* path_p = 0;
-  GtkTreeViewDropPosition drop_pos;
-  if (gtk_tree_view_get_dest_row_at_pos(instance_p->folder_tree_view_p, x, y, &path_p, &drop_pos)
-      && path_p) {
-
-    // wrap path_p in a TreePathScopedHandle object
-    // so we do not need to delete it by hand
-    TreePathScopedHandle path_h(path_p);
-    path_p = 0; // we don't need it any more
-    if (instance_p->drag_is_fax) {
-      gtk_tree_view_set_drag_dest_row(instance_p->folder_tree_view_p, path_h,
-				      GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
-    }
-    else {
-      gtk_tree_view_set_drag_dest_row(instance_p->folder_tree_view_p, path_h, drop_pos);
-    }
-    gdk_drag_status(context_p, GDK_ACTION_MOVE, time);
-    return_val = true;
-  }
-  else {
-    gdk_drag_status(context_p, GdkDragAction(0), time);
-    return_val = false;
-  }
-  return return_val;
-}
-
-gboolean FaxListManagerCB::folder_tree_view_motion_notify(GtkWidget*,
-							  GdkEventMotion*, void* data) {
-
-  FaxListManager* instance_p = static_cast<FaxListManager*>(data);
-
-  int x, y;
-  gtk_widget_get_pointer(GTK_WIDGET(instance_p->folder_tree_view_p), &x, &y);
-  if (x >= 0 && y >= 0) {
-    bool result = gtk_tree_view_get_dest_row_at_pos(instance_p->folder_tree_view_p,
-						    x, y, 0, 0);
-    if (result && !instance_p->folder_drag_source_enabled) {
-      gtk_drag_source_set(GTK_WIDGET(instance_p->folder_tree_view_p),
-			  GDK_MODIFIER_MASK, instance_p->target_array,
-			  instance_p->target_size, GDK_ACTION_MOVE);
-      instance_p->folder_drag_source_enabled = true;
-    }      
-    else if (!result && instance_p->folder_drag_source_enabled) {
-      gtk_drag_source_unset(GTK_WIDGET(instance_p->folder_tree_view_p));
-      instance_p->folder_drag_source_enabled = false;
-    }
-  }
-  return false;
-}
-
-gboolean FaxListManagerCB::fax_tree_view_motion_notify(GtkWidget*,
-						       GdkEventMotion*, void* data) {
-
-  FaxListManager* instance_p = static_cast<FaxListManager*>(data);
-
-  int x, y;
-  gtk_widget_get_pointer(GTK_WIDGET(instance_p->fax_tree_view_p), &x, &y);
-
-  if (x >= 0 && y >= 0) {
-    bool result = gtk_tree_view_get_dest_row_at_pos(instance_p->fax_tree_view_p,
-						    x, y, 0, 0);
-    if (result && !instance_p->fax_drag_source_enabled) {
-      gtk_drag_source_set(GTK_WIDGET(instance_p->fax_tree_view_p),
-			  GDK_MODIFIER_MASK, instance_p->target_array,
-			  instance_p->target_size, GDK_ACTION_MOVE);
-      instance_p->fax_drag_source_enabled = true;
-    }
-    else if (!result && instance_p->fax_drag_source_enabled) {
-      gtk_drag_source_unset(GTK_WIDGET(instance_p->fax_tree_view_p));
-      instance_p->fax_drag_source_enabled = false;
-    }
-  }
-  return false;
-}
-
-gboolean FaxListManagerCB::fax_tree_view_mouse_click(GtkWidget*,
-						      GdkEventButton* event_p, void* data) {
-  
-  FaxListManager* instance_p = static_cast<FaxListManager*>(data);
-
-  // we can use fax_drag_source_enabled as a proxy for whether the pointer
-  // is over a selectable (and so selected) row in the tree view when
-  // a double click occurs
-  if (event_p->type == GDK_2BUTTON_PRESS
-      && instance_p->fax_drag_source_enabled) {
-    instance_p->fax_double_click_notify();
-  }
-  return false;
-}
-
-
-void FaxListManagerCB::drag_data_get(GtkWidget*, GdkDragContext*,
-				     GtkSelectionData* selection_data_p,
-				     guint, guint, void*) {
-
-  // all we do in this method is to set up an identification string (DRAG_KEY_STRING)
-  // so that drags not originating in this program can be ignored.  The data which
-  // is dragged is placed in FaxListManager::folder_drag_source_row_iter in method
-  // FaxListManagerCB::folder_tree_view_drag_begin() (for a folder drag) and in
-  // FaxListManager::fax_drag_source_row_refs in method
-  // FaxListManagerCB::fax_tree_view_drag_begin() (for a fax drag), rather than in
-  // this method.  We can therefore use this method for drags from both the folder tree
-  // view and the fax tree view.  The method FaxListManagerCB::drag_data_received()
-  // tests for this identification string when a drag is received by the folder tree
-  // view (the fax tree view does not receive drags, it only sources them, whereas the
-  // folder tree view both receives and sources drags).
-
-  const std::string message(DRAG_KEY_STRING);
-
-#if GTK_CHECK_VERSION(2,14,0)
-  gtk_selection_data_set(selection_data_p, gtk_selection_data_get_target(selection_data_p),
-			 8, (const guchar*)message.c_str(), message.size() + 1);
-#else
-  gtk_selection_data_set(selection_data_p, selection_data_p->target, 8,
-			 (const guchar*)message.c_str(), message.size() + 1);
+      gtk_drag_set_icon_pixmap(context_p, colormap_p, icon_h, 0, x_pos, cell_height/2);
 #endif
-}
-
-void FaxListManagerCB::drag_data_received(GtkWidget*, GdkDragContext* context_p,
-					  gint x, gint y, GtkSelectionData* selection_data_p,
-					  guint, guint time, void* data) {
-
-  FaxListManager* instance_p = static_cast<FaxListManager*>(data);
-
-  bool success = false;
-  std::string message;
-
-#if GTK_CHECK_VERSION(2,14,0)
-  if(gtk_selection_data_get_length(selection_data_p) >= 0
-     && gtk_selection_data_get_format(selection_data_p) == 8) {
-    message = (const char*)gtk_selection_data_get_data(selection_data_p);
+    }
   }
-#else
-  if(selection_data_p->length >= 0 && selection_data_p->format == 8) {
-    message = (const char*)selection_data_p->data;
-  }
-#endif
 
-  if (message.find(DRAG_KEY_STRING) != std::string::npos) {
-    
+  static gboolean drag_motion(GdkDragContext* context_p,
+			      gint x, gint y,
+			      guint time,
+			      void* data) {
+
+    FaxListManager* instance_p = static_cast<FaxListManager*>(data);
+
+    bool return_val;
     GtkTreePath* path_p = 0;
     GtkTreeViewDropPosition drop_pos;
     if (gtk_tree_view_get_dest_row_at_pos(instance_p->folder_tree_view_p, x, y, &path_p, &drop_pos)
 	&& path_p) {
+
       // wrap path_p in a TreePathScopedHandle object
       // so we do not need to delete it by hand
       TreePathScopedHandle path_h(path_p);
       path_p = 0; // we don't need it any more
-      GtkTreeIter iter;
-      if (gtk_tree_model_get_iter(instance_p->folder_tree_store_h, &iter, path_h)) {
-	if (instance_p->drag_is_fax) instance_p->move_fax(&iter);
-	else instance_p->move_folder(&iter, drop_pos);
-	success = true;
+      if (instance_p->drag_is_fax) {
+	gtk_tree_view_set_drag_dest_row(instance_p->folder_tree_view_p, path_h,
+					GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
       }
-      else write_error("Invalid iterator in FaxListManagerCB::drag_data_received()\n");
+      else {
+	gtk_tree_view_set_drag_dest_row(instance_p->folder_tree_view_p, path_h, drop_pos);
+      }
+      gdk_drag_status(context_p, GDK_ACTION_MOVE, time);
+      return_val = true;
     }
+    else {
+      gdk_drag_status(context_p, GdkDragAction(0), time);
+      return_val = false;
+    }
+    return return_val;
   }
-  gtk_drag_finish(context_p, success, false, time);
-}
 
-void FaxListManagerCB::fax_selected(GtkTreeSelection*, void* data) {
-  static_cast<FaxListManager*>(data)->selection_notify();
-}
+  static gboolean folder_tree_view_motion_notify(GdkEventMotion* event_p,
+						 void* data) {
 
-void FaxListManagerCB::folder_selected(GtkTreeSelection*, void* data) {
-  FaxListManager* instance_p = static_cast<FaxListManager*>(data);
+    FaxListManager* instance_p = static_cast<FaxListManager*>(data);
 
-  // display_faxes() needs to be called first before selection_notify
-  // emitter is emitted so that the correct fax tree model is in
-  // position before the functor connected to the emitter is executed
-  instance_p->display_faxes();
-  instance_p->selection_notify();
-}
-
-void FaxListManagerCB::toggle_sort_type(GtkTreeViewColumn*, void* data) {
-  FaxListManager* instance_p = static_cast<FaxListManager*>(data);
-  if (instance_p->list_sort_type[instance_p->mode] == GTK_SORT_ASCENDING) 
-    instance_p->list_sort_type[instance_p->mode] = GTK_SORT_DESCENDING;
-  else instance_p->list_sort_type[instance_p->mode] = GTK_SORT_ASCENDING;
-
-  FolderRowToFaxModelMap::const_iterator iter;
-  for (iter = instance_p->folder_row_to_fax_model_map.begin();
-       iter != instance_p->folder_row_to_fax_model_map.end();
-       ++iter) {
-    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(iter->second.get()),
-					 FaxListModelColumns::name,
-					 instance_p->list_sort_type[instance_p->mode]);
+    gint x, y;
+    gtk_tree_view_convert_bin_window_to_widget_coords(instance_p->folder_tree_view_p,
+						      event_p->x, event_p->y,
+						      &x, &y);
+    if (x >= 0 && y >= 0) {
+      bool result = gtk_tree_view_get_dest_row_at_pos(instance_p->folder_tree_view_p,
+						      x, y, 0, 0);
+      if (result && !instance_p->folder_drag_source_enabled) {
+	gtk_drag_source_set(GTK_WIDGET(instance_p->folder_tree_view_p),
+			    GDK_MODIFIER_MASK, instance_p->target_array,
+			    instance_p->target_size, GDK_ACTION_MOVE);
+	instance_p->folder_drag_source_enabled = true;
+      }      
+      else if (!result && instance_p->folder_drag_source_enabled) {
+	gtk_drag_source_unset(GTK_WIDGET(instance_p->folder_tree_view_p));
+	instance_p->folder_drag_source_enabled = false;
+      }
+    }
+    return false;
   }
-  // this doesn't control the sort order (the call to gtk_tree_sortable_set_sort_column_id()
-  // did that) - it just sets the direction that the arrow in the tree view shows!
-  gtk_tree_view_column_set_sort_order(instance_p->fax_date_column_p,
-				      instance_p->list_sort_type[instance_p->mode]);  
-}
 
-} // anonymous namespace
+  static gboolean fax_tree_view_motion_notify(GdkEventMotion* event_p,
+					      void* data) {
 
+    FaxListManager* instance_p = static_cast<FaxListManager*>(data);
 
+    gint x, y;
+    gtk_tree_view_convert_bin_window_to_widget_coords(instance_p->fax_tree_view_p,
+						      event_p->x, event_p->y,
+						      &x, &y);
+    if (x >= 0 && y >= 0) {
+      bool result = gtk_tree_view_get_dest_row_at_pos(instance_p->fax_tree_view_p,
+						      x, y, 0, 0);
+      if (result && !instance_p->fax_drag_source_enabled) {
+	gtk_drag_source_set(GTK_WIDGET(instance_p->fax_tree_view_p),
+			    GDK_MODIFIER_MASK, instance_p->target_array,
+			    instance_p->target_size, GDK_ACTION_MOVE);
+	instance_p->fax_drag_source_enabled = true;
+      }
+      else if (!result && instance_p->fax_drag_source_enabled) {
+	gtk_drag_source_unset(GTK_WIDGET(instance_p->fax_tree_view_p));
+	instance_p->fax_drag_source_enabled = false;
+      }
+    }
+    return false;
+  }
+
+  static gboolean fax_tree_view_mouse_click(GdkEventButton* event_p,
+					    void* data) {
+  
+    FaxListManager* instance_p = static_cast<FaxListManager*>(data);
+
+    // we can use fax_drag_source_enabled as a proxy for whether the pointer
+    // is over a selectable (and so selected) row in the tree view when
+    // a double click occurs
+    if (event_p->type == GDK_2BUTTON_PRESS
+	&& instance_p->fax_drag_source_enabled) {
+      instance_p->fax_double_click_notify();
+    }
+    return false;
+  }
+
+  static void drag_data_get(GtkSelectionData* selection_data_p) {
+
+    // all we do in this method is to set up an identification string (DRAG_KEY_STRING)
+    // so that drags not originating in this program can be ignored.  The data which
+    // is dragged is placed in FaxListManager::folder_drag_source_row_iter in method
+    // FaxListManager::CB::folder_tree_view_drag_begin() (for a folder drag) and in
+    // FaxListManager::fax_drag_source_row_refs in method
+    // FaxListManager::CB::fax_tree_view_drag_begin() (for a fax drag), rather than in
+    // this method.  We can therefore use this method for drags from both the folder tree
+    // view and the fax tree view.  The method FaxListManager::CB::drag_data_received()
+    // tests for this identification string when a drag is received by the folder tree
+    // view (the fax tree view does not receive drags, it only sources them, whereas the
+    // folder tree view both receives and sources drags).
+
+    const std::string message(DRAG_KEY_STRING);
+
+#if GTK_CHECK_VERSION(2,14,0)
+    gtk_selection_data_set(selection_data_p, gtk_selection_data_get_target(selection_data_p),
+			   8, (const guchar*)message.c_str(), message.size() + 1);
+#else
+    gtk_selection_data_set(selection_data_p, selection_data_p->target, 8,
+			   (const guchar*)message.c_str(), message.size() + 1);
+#endif
+  }
+
+  static void drag_data_received(GdkDragContext* context_p,
+				 gint x, gint y,
+				 GtkSelectionData* selection_data_p,
+				 guint time,
+				 void* data) {
+
+    FaxListManager* instance_p = static_cast<FaxListManager*>(data);
+
+    bool success = false;
+    std::string message;
+
+#if GTK_CHECK_VERSION(2,14,0)
+    if(gtk_selection_data_get_length(selection_data_p) >= 0
+       && gtk_selection_data_get_format(selection_data_p) == 8) {
+      message = (const char*)gtk_selection_data_get_data(selection_data_p);
+    }
+#else
+    if(selection_data_p->length >= 0 && selection_data_p->format == 8) {
+      message = (const char*)selection_data_p->data;
+    }
+#endif
+
+    if (message.find(DRAG_KEY_STRING) != std::string::npos) {
+    
+      GtkTreePath* path_p = 0;
+      GtkTreeViewDropPosition drop_pos;
+      if (gtk_tree_view_get_dest_row_at_pos(instance_p->folder_tree_view_p, x, y, &path_p, &drop_pos)
+	  && path_p) {
+	// wrap path_p in a TreePathScopedHandle object
+	// so we do not need to delete it by hand
+	TreePathScopedHandle path_h(path_p);
+	path_p = 0; // we don't need it any more
+	GtkTreeIter iter;
+	if (gtk_tree_model_get_iter(instance_p->folder_tree_store_h, &iter, path_h)) {
+	  if (instance_p->drag_is_fax) instance_p->move_fax(&iter);
+	  else instance_p->move_folder(&iter, drop_pos);
+	  success = true;
+	}
+	else write_error("Invalid iterator in FaxListManager::CB::drag_data_received()\n");
+      }
+    }
+    gtk_drag_finish(context_p, success, false, time);
+  }
+
+  static void fax_selected(void* data) {
+    static_cast<FaxListManager*>(data)->selection_notify();
+  }
+
+  static void folder_selected(void* data) {
+    FaxListManager* instance_p = static_cast<FaxListManager*>(data);
+
+    // display_faxes() needs to be called first before selection_notify
+    // emitter is emitted so that the correct fax tree model is in
+    // position before the functor connected to the emitter is executed
+    instance_p->display_faxes();
+    instance_p->selection_notify();
+  }
+
+  static void toggle_sort_type(void* data) {
+    FaxListManager* instance_p = static_cast<FaxListManager*>(data);
+    if (instance_p->list_sort_type[instance_p->mode] == GTK_SORT_ASCENDING) 
+      instance_p->list_sort_type[instance_p->mode] = GTK_SORT_DESCENDING;
+    else instance_p->list_sort_type[instance_p->mode] = GTK_SORT_ASCENDING;
+
+    FolderRowToFaxModelMap::const_iterator iter;
+    for (iter = instance_p->folder_row_to_fax_model_map.begin();
+	 iter != instance_p->folder_row_to_fax_model_map.end();
+	 ++iter) {
+      gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(iter->second.get()),
+					   FaxListModelColumns::name,
+					   instance_p->list_sort_type[instance_p->mode]);
+    }
+    // this doesn't control the sort order (the call to gtk_tree_sortable_set_sort_column_id()
+    // did that) - it just sets the direction that the arrow in the tree view shows!
+    gtk_tree_view_column_set_sort_order(instance_p->fax_date_column_p,
+					instance_p->list_sort_type[instance_p->mode]);  
+  }
+};
+
+extern "C" {
+
+  static void folder_tree_view_drag_begin_cb(GtkWidget*,
+					     GdkDragContext* context_p,
+					     void* data) {
+    FaxListManager::CB::folder_tree_view_drag_begin(context_p, data);
+  }
+
+  static void fax_tree_view_drag_begin_cb(GtkWidget*,
+					  GdkDragContext* context_p,
+					  void* data) {
+    FaxListManager::CB::fax_tree_view_drag_begin(context_p, data);
+  }
+
+  static gboolean drag_motion_cb(GtkWidget*,
+				 GdkDragContext* context_p,
+				 gint x, gint y,
+				 guint time,
+				 void* data) {
+    return FaxListManager::CB::drag_motion(context_p, x, y, time, data);
+  }
+
+  static gboolean folder_tree_view_motion_notify_cb(GtkWidget*,
+						    GdkEventMotion* event_p,
+						    void* data) {
+    return FaxListManager::CB::folder_tree_view_motion_notify(event_p, data);
+  }
+
+  static gboolean fax_tree_view_motion_notify_cb(GtkWidget*,
+					      GdkEventMotion* event_p,
+					      void* data) {
+    return FaxListManager::CB::fax_tree_view_motion_notify(event_p, data);
+  }
+
+  static gboolean fax_tree_view_mouse_click_cb(GtkWidget*,
+					       GdkEventButton* event_p,
+					       void* data) {
+    return FaxListManager::CB::fax_tree_view_mouse_click(event_p, data);
+  }
+
+  static void drag_data_get_cb(GtkWidget*,
+			       GdkDragContext*,
+			       GtkSelectionData* selection_data_p,
+			       guint,
+			       guint,
+			       void*) {
+    FaxListManager::CB::drag_data_get(selection_data_p);
+  }
+
+  static void drag_data_received_cb(GtkWidget*,
+				    GdkDragContext* context_p,
+				    gint x, gint y,
+				    GtkSelectionData* selection_data_p,
+				    guint,
+				    guint time,
+				    void* data) {
+    FaxListManager::CB::drag_data_received(context_p, x, y, selection_data_p, time, data);
+  }
+
+  static void fax_selected_cb(GtkTreeSelection*,
+			      void* data) {
+    FaxListManager::CB::fax_selected(data);
+  }
+
+  static void folder_selected_cb(GtkTreeSelection*,
+				 void* data) {
+    FaxListManager::CB::folder_selected(data);
+  }
+
+  static void toggle_sort_type_cb(GtkTreeViewColumn*,
+			       void* data) {
+    FaxListManager::CB::toggle_sort_type(data);
+  }
+
+} // extern "C"
 
 FaxListManager::FaxListManager(FaxListEnum::Mode mode_):
                                                 mode(mode_),
@@ -423,7 +519,7 @@ FaxListManager::FaxListManager(FaxListEnum::Mode mode_):
   // make the Date column clickable
   gtk_tree_view_column_set_clickable(fax_date_column_p, true);
   g_signal_connect(G_OBJECT(fax_date_column_p), "clicked",
-		   G_CALLBACK(FaxListManagerCB::toggle_sort_type), this);
+		   G_CALLBACK(toggle_sort_type_cb), this);
   gtk_tree_view_append_column(fax_tree_view_p, fax_date_column_p);
 
 
@@ -446,46 +542,51 @@ FaxListManager::FaxListManager(FaxListEnum::Mode mode_):
   // now provide drag and drop from the fax tree view to the folder tree view
   // first connect slots to ensure that a drag only starts with a valid row to drag
   g_signal_connect(G_OBJECT(folder_tree_view_p), "motion_notify_event",
-		   G_CALLBACK(FaxListManagerCB::folder_tree_view_motion_notify), this);
+		   G_CALLBACK(folder_tree_view_motion_notify_cb), this);
   g_signal_connect(G_OBJECT(fax_tree_view_p), "motion_notify_event",
-		   G_CALLBACK(FaxListManagerCB::fax_tree_view_motion_notify), this);
+		   G_CALLBACK(fax_tree_view_motion_notify_cb), this);
 
   // connect slots to handle the drag source in the fax tree view
   g_signal_connect(G_OBJECT(fax_tree_view_p), "drag_data_get",
-		   G_CALLBACK(FaxListManagerCB::drag_data_get), this);
+		   G_CALLBACK(drag_data_get_cb), this);
   g_signal_connect(G_OBJECT(fax_tree_view_p), "drag_begin",
-		   G_CALLBACK(FaxListManagerCB::fax_tree_view_drag_begin), this);
+		   G_CALLBACK(fax_tree_view_drag_begin_cb), this);
 
   // connect slots to handle the drag source in the folder tree view
   g_signal_connect(G_OBJECT(folder_tree_view_p), "drag_data_get",
-		   G_CALLBACK(FaxListManagerCB::drag_data_get), this);
+		   G_CALLBACK(drag_data_get_cb), this);
   g_signal_connect(G_OBJECT(folder_tree_view_p), "drag_begin",
-		   G_CALLBACK(FaxListManagerCB::folder_tree_view_drag_begin), this);
+		   G_CALLBACK(folder_tree_view_drag_begin_cb), this);
 
   // connect slots to handle the drag destination in the folder tree view
   g_signal_connect(G_OBJECT(folder_tree_view_p), "drag_motion",
-		   G_CALLBACK(FaxListManagerCB::drag_motion), this);
+		   G_CALLBACK(drag_motion_cb), this);
   gtk_drag_dest_set(GTK_WIDGET(folder_tree_view_p), GTK_DEST_DEFAULT_ALL,
 		    target_array, target_size, GDK_ACTION_MOVE);
   g_signal_connect(G_OBJECT(folder_tree_view_p), "drag_data_received",
-		   G_CALLBACK(FaxListManagerCB::drag_data_received), this);
+		   G_CALLBACK(drag_data_received_cb), this);
 
   // handle selection of a row in the tree views
   selection_p = gtk_tree_view_get_selection(fax_tree_view_p);
   g_signal_connect(G_OBJECT(selection_p), "changed",
-		   G_CALLBACK(FaxListManagerCB::fax_selected), this);
+		   G_CALLBACK(fax_selected_cb), this);
   selection_p = gtk_tree_view_get_selection(folder_tree_view_p);
   g_signal_connect(G_OBJECT(selection_p), "changed",
-		   G_CALLBACK(FaxListManagerCB::folder_selected), this);
+		   G_CALLBACK(folder_selected_cb), this);
 
   // capture a double click on the fax tree view for viewing a fax
   gtk_widget_add_events(GTK_WIDGET(fax_tree_view_p), GDK_BUTTON_PRESS_MASK);
   g_signal_connect(G_OBJECT(fax_tree_view_p), "button_press_event",
-		   G_CALLBACK(FaxListManagerCB::fax_tree_view_mouse_click), this);
+		   G_CALLBACK(fax_tree_view_mouse_click_cb), this);
 
   // set a timeout to call write_path() at 5 minute intervals
+#if GLIB_CHECK_VERSION(2,14,0)
+  timer_tag = start_timeout_seconds(WRITE_PATH_TIMER_INTERVAL/1000,
+				    Callback::make(*this, &FaxListManager::write_path_timer_cb));
+#else
   timer_tag = start_timeout(WRITE_PATH_TIMER_INTERVAL,
 			    Callback::make(*this, &FaxListManager::write_path_timer_cb));
+#endif
 }
 
 FaxListManager::~FaxListManager(void) {
