@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2005 and 2009 Chris Vine
+/* Copyright (C) 2001 to 2005, 2009 and 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -36,14 +36,13 @@ COPYING distributed with the source files.
 int FileListDialog::is_file_list = 0;
 
 
+namespace {
 namespace ModelColumns {
   enum {file_name, cols_num};
-}
+} // namespace ModelColumns
+} // anonymous namespace
 
-
-namespace { // callbacks for internal use only
-
-void FileListDialogCB::file_list_button_clicked(GtkWidget* widget_p, void* data) {
+void FileListDialogCB::file_list_dialog_button_clicked(GtkWidget* widget_p, void* data) {
   FileListDialog* instance_p = static_cast<FileListDialog*>(data);
 
   if (widget_p == instance_p->ok_button_p) {
@@ -69,12 +68,12 @@ void FileListDialogCB::file_list_button_clicked(GtkWidget* widget_p, void* data)
     instance_p->remove_file_prompt();
   }
   else {
-    write_error("Callback error in FileListDialogCB::file_list_button_clicked()\n");
+    write_error("Callback error in FileListDialogCB::file_list_dialog_button_clicked()\n");
     instance_p->close();
   }
 }
 
-void FileListDialogCB::file_list_set_buttons(GtkTreeSelection*, void* data) {
+void FileListDialogCB::file_list_dialog_set_buttons(GtkTreeSelection*, void* data) {
   FileListDialog* instance_p = static_cast<FileListDialog*>(data);
 
   // see if anything is selected
@@ -94,8 +93,6 @@ void FileListDialogCB::file_list_set_buttons(GtkTreeSelection*, void* data) {
   }
 }
 
-}  // anonymous namespace
-
 
 FileListDialog::FileListDialog(const int standard_size_, GtkWindow* parent_p):
                                WinBase(gettext("efax-gtk: File list"),
@@ -248,25 +245,25 @@ FileListDialog::FileListDialog(const int standard_size_, GtkWindow* parent_p):
   gtk_box_pack_start(GTK_BOX(file_list_box_p), GTK_WIDGET(table_p), true, true, 0);
 
   g_signal_connect(G_OBJECT(ok_button_p), "clicked",
-		   G_CALLBACK(FileListDialogCB::file_list_button_clicked), this);
+		   G_CALLBACK(FileListDialogCB::file_list_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(cancel_button_p), "clicked",
-		   G_CALLBACK(FileListDialogCB::file_list_button_clicked), this);
+		   G_CALLBACK(FileListDialogCB::file_list_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(up_button_p), "clicked",
-		   G_CALLBACK(FileListDialogCB::file_list_button_clicked), this);
+		   G_CALLBACK(FileListDialogCB::file_list_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(down_button_p), "clicked",
-		   G_CALLBACK(FileListDialogCB::file_list_button_clicked), this);
+		   G_CALLBACK(FileListDialogCB::file_list_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(add_button_p), "clicked",
-		   G_CALLBACK(FileListDialogCB::file_list_button_clicked), this);
+		   G_CALLBACK(FileListDialogCB::file_list_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(view_button_p), "clicked",
-		   G_CALLBACK(FileListDialogCB::file_list_button_clicked), this);
+		   G_CALLBACK(FileListDialogCB::file_list_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(remove_button_p), "clicked",
-		   G_CALLBACK(FileListDialogCB::file_list_button_clicked), this);
+		   G_CALLBACK(FileListDialogCB::file_list_dialog_button_clicked), this);
 
 
   // now connect up the signal which indicates a selection has been made
   selection_p = gtk_tree_view_get_selection(tree_view_p);
   g_signal_connect(G_OBJECT(selection_p), "changed",
-		   G_CALLBACK(FileListDialogCB::file_list_set_buttons), this);
+		   G_CALLBACK(FileListDialogCB::file_list_dialog_set_buttons), this);
 
 #if GTK_CHECK_VERSION(2,20,0)
   gtk_widget_set_can_default(ok_button_p, true);
