@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2005 Chris Vine
+/* Copyright (C) 2001 to 2005 and 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -19,16 +19,24 @@ COPYING distributed with the source files.
 
 int HelpDialog::is_helpfile = 0;
 
-namespace { // callbacks for internal use only
-
-void HelpDialogCB::help_button_clicked(GtkWidget*, void* data) {
+void HelpDialogCB::help_dialog_button_clicked(GtkWidget*, void* data) {
   static_cast<HelpDialog*>(data)->close();
 }
 
-gboolean HelpDialogCB::help_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
+gboolean HelpDialogCB::help_dialog_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
   HelpDialog* instance_p = static_cast<HelpDialog*>(data);
   int keycode = event_p->keyval;
 
+#if GTK_CHECK_VERSION(2,91,0)
+  if (keycode == GDK_KEY_Home || keycode == GDK_KEY_End
+      || keycode == GDK_KEY_Up || keycode == GDK_KEY_Down
+      || keycode == GDK_KEY_Page_Up || keycode == GDK_KEY_Page_Down) {
+    gtk_widget_event(gtk_notebook_get_nth_page(instance_p->notebook_p,
+					       gtk_notebook_get_current_page(instance_p->notebook_p)),
+		     (GdkEvent*)event_p);
+    return true;    // stop processing here
+  }
+#else
   if (keycode == GDK_Home || keycode == GDK_End
       || keycode == GDK_Up || keycode == GDK_Down
       || keycode == GDK_Page_Up || keycode == GDK_Page_Down) {
@@ -37,11 +45,10 @@ gboolean HelpDialogCB::help_key_press_event(GtkWidget*, GdkEventKey* event_p, vo
 		     (GdkEvent*)event_p);
     return true;    // stop processing here
   }
+#endif
   return false;     // pass on the key event
 }
 
-}  // anonymous namespace
-
 HelpDialog::HelpDialog(const int standard_size): 
                              WinBase(gettext("efax-gtk: Help"),
 				     prog_config.window_icon_h) {
@@ -109,9 +116,9 @@ HelpDialog::HelpDialog(const int standard_size):
   gtk_box_pack_start(vbox_p, button_box_p, false, false, standard_size/3);
 
   g_signal_connect(G_OBJECT(close_button_p), "clicked",
-		   G_CALLBACK(HelpDialogCB::help_button_clicked), this);
+		   G_CALLBACK(HelpDialogCB::help_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(get_win()), "key_press_event",
-		   G_CALLBACK(HelpDialogCB::help_key_press_event), this);
+		   G_CALLBACK(HelpDialogCB::help_dialog_key_press_event), this);
 
   gtk_container_add(GTK_CONTAINER(get_win()), GTK_WIDGET(vbox_p));
 
