@@ -642,6 +642,16 @@ bool operator==(const FunctorArg<T>& f1, const FunctorArg<T>& f2) {
   return (f1.cb_s.get() == f2.cb_s.get());
 }
 
+/**
+ * This comparison operator does not throw.
+ *
+ * Since 1.2.5
+ */
+template <class T>
+bool operator!=(const FunctorArg<T>& f1, const FunctorArg<T>& f2) {
+  return !(f1 == f2);
+}
+
 // we must use std::less rather than the < built-in operator for
 // pointers to objects not within the same array or object: "For
 // templates greater, less, greater_equal, and less_equal, the
@@ -663,6 +673,16 @@ bool operator==(const SafeFunctorArg<T>& f1, const SafeFunctorArg<T>& f2) {
   return (f1.cb_s.get() == f2.cb_s.get());
 }
 
+/**
+ * This comparison operator does not throw.
+ *
+ * Since 1.2.5
+ */
+template <class T>
+bool operator!=(const SafeFunctorArg<T>& f1, const SafeFunctorArg<T>& f2) {
+  return !(f1 == f2);
+}
+
 /**
  * This comparison operator does not throw.
  */
@@ -743,7 +763,8 @@ public:
   void operator()(FreeArg arg) const {if (cb_s.get()) cb_s->dispatch(arg);}
 
 /**
- * This comparison operator does not throw.
+ * This comparison operator does not throw (this friend function is
+ * also called by operator!=<>() from version 1.2.5).
  */
   friend bool operator== <>(const FunctorArg&, const FunctorArg&);
 /**
@@ -842,7 +863,8 @@ public:
   void operator()(FreeArg arg) const {if (cb_s.get()) cb_s->dispatch(arg);}
 
 /**
- * This comparison operator does not throw.
+ * This comparison operator does not throw (this friend function is
+ * also called by operator!=<>() from version 1.2.5).
  */
   friend bool operator== <>(const SafeFunctorArg&, const SafeFunctorArg&);
 /**
