@@ -1,4 +1,4 @@
-/* Copyright (C) 2007 and 2009 Chris Vine
+/* Copyright (C) 2007 and 2010 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -61,98 +61,141 @@ std::string Cgu::TextPrintManager::default_font_family("Mono");
 int Cgu::TextPrintManager::default_font_size = 10;
 
 #ifndef DOXYGEN_PARSING
-namespace { // callbacks for internal use only
+namespace Cgu {
 
-void CguTextPrintManagerCB::text_print_begin_print(GtkPrintOperation* print_operation_p,
-						   GtkPrintContext* context_p,
-						   void* data) {
-  static_cast<Cgu::TextPrintManager*>(data)->begin_print_impl(print_operation_p, context_p);
-}
+class TextPrintManager::CB {
+public:
 
-void CguTextPrintManagerCB::text_print_draw_page(GtkPrintOperation* print_operation_p,
-						 GtkPrintContext* context_p,
-						 gint page_nr,
-						 void* data) {
-  static_cast<Cgu::TextPrintManager*>(data)->draw_page_impl(print_operation_p, context_p, page_nr);
-}
+  static void begin_print(GtkPrintOperation* print_operation_p,
+			  GtkPrintContext* context_p,
+			  void* data) {
+    static_cast<Cgu::TextPrintManager*>(data)->begin_print_impl(print_operation_p, context_p);
+  }
 
-void CguTextPrintManagerCB::text_print_done(GtkPrintOperation* print_operation_p,
-					    GtkPrintOperationResult result,
-					    void* data) {
+  static void draw_page(GtkPrintOperation* print_operation_p,
+			GtkPrintContext* context_p,
+			gint page_nr,
+			void* data) {
+    static_cast<Cgu::TextPrintManager*>(data)->draw_page_impl(print_operation_p, context_p, page_nr);
+  }
 
-  Cgu::TextPrintManager* instance_p = static_cast<Cgu::TextPrintManager*>(data);
-  if (result == GTK_PRINT_OPERATION_RESULT_ERROR) {
-    GError* error_p = 0;
-    gtk_print_operation_get_error(print_operation_p, &error_p);
-    if (error_p) {
-      Cgu::GerrorScopedHandle handle_h(error_p);
-      g_warning("CguTextPrintManagerCB::text_print_done(): %s\n", error_p->message);
+  static void done(GtkPrintOperation* print_operation_p,
+		   GtkPrintOperationResult result,
+		   void* data) {
+
+    Cgu::TextPrintManager* instance_p = static_cast<Cgu::TextPrintManager*>(data);
+    if (result == GTK_PRINT_OPERATION_RESULT_ERROR) {
+      GError* error_p = 0;
+      gtk_print_operation_get_error(print_operation_p, &error_p);
+      if (error_p) {
+	Cgu::GerrorScopedHandle handle_h(error_p);
+	g_warning("CguTextPrintManagerCB::text_print_done(): %s\n", error_p->message);
+      }
     }
-  }
-  else if (result == GTK_PRINT_OPERATION_RESULT_APPLY) {
-    // save the print settings
-    Cgu::TextPrintManager::print_settings_h =
-      Cgu::GobjHandle<GtkPrintSettings>(gtk_print_operation_get_print_settings(print_operation_p));
-    // and take ownership of the GtkPrintSettings object
-    g_object_ref(G_OBJECT(Cgu::TextPrintManager::print_settings_h.get()));
-  }
-  else if (result == GTK_PRINT_OPERATION_RESULT_CANCEL
-	   && instance_p->parent_p
-	   && !instance_p->cancelled_when_drawing) {
-    // the parent will not be set sensitive if TextPrintManager::begin_print_impl()
-    // has not been entered because the print job was cancelled from the print dialog
-    gtk_widget_set_sensitive(GTK_WIDGET(instance_p->parent_p), true);
-  }
-  { // scope block for mutex lock
-    Cgu::Thread::Mutex::Lock lock(instance_p->mutex);
-    instance_p->ready = true;
-  }
-  instance_p->unref();
-}
-
-void CguTextPrintManagerCB::text_print_page_setup_done(GtkPageSetup* page_setup_p,
-						       void* data) {
-
-  // set some sane margins if we are not using a custom paper/margin size
-  if (page_setup_p) {
-    if (!gtk_paper_size_is_custom (gtk_page_setup_get_paper_size (page_setup_p))) {
-      gtk_page_setup_set_top_margin(page_setup_p, Cgu::STANDARD_MARGINS, GTK_UNIT_MM);
-      gtk_page_setup_set_bottom_margin(page_setup_p, Cgu::STANDARD_MARGINS, GTK_UNIT_MM);
-      gtk_page_setup_set_left_margin(page_setup_p, Cgu::STANDARD_MARGINS, GTK_UNIT_MM);
-      gtk_page_setup_set_right_margin(page_setup_p, Cgu::STANDARD_MARGINS, GTK_UNIT_MM);
+    else if (result == GTK_PRINT_OPERATION_RESULT_APPLY) {
+      // save the print settings
+      Cgu::TextPrintManager::print_settings_h =
+	Cgu::GobjHandle<GtkPrintSettings>(gtk_print_operation_get_print_settings(print_operation_p));
+      // and take ownership of the GtkPrintSettings object
+      g_object_ref(G_OBJECT(Cgu::TextPrintManager::print_settings_h.get()));
     }
-    // The documentation is unclear whether we get a new GtkPageSetup
-    // object back if an existing GtkPageSetup object was passed to
-    // gtk_print_run_page_setup_dialog_async().  However this is safe
-    // even if we get the same object back again - see GobjHandle
-    // implementation
-    Cgu::TextPrintManager::page_setup_h = Cgu::GobjHandle<GtkPageSetup>(page_setup_p);
+    else if (result == GTK_PRINT_OPERATION_RESULT_CANCEL
+	     && instance_p->parent_p
+	     && !instance_p->cancelled_when_drawing) {
+      // the parent will not be set sensitive if TextPrintManager::begin_print_impl()
+      // has not been entered because the print job was cancelled from the print dialog
+      gtk_widget_set_sensitive(GTK_WIDGET(instance_p->parent_p), true);
+    }
+    { // scope block for mutex lock
+      Cgu::Thread::Mutex::Lock lock(instance_p->mutex);
+      instance_p->ready = true;
+    }
+    instance_p->unref();
   }
-  if (data) gtk_widget_set_sensitive(GTK_WIDGET(data), true);
-}
 
-GObject* CguTextPrintManagerCB::text_print_create_custom_widget(GtkPrintOperation* print_operation_p,
-								void* data) {
+  static void page_setup_done(GtkPageSetup* page_setup_p,
+			      void* data) {
 
-  return static_cast<Cgu::TextPrintManager*>(data)->create_custom_widget_impl(print_operation_p);
-}
+    // set some sane margins if we are not using a custom paper/margin size
+    if (page_setup_p) {
+      if (!gtk_paper_size_is_custom (gtk_page_setup_get_paper_size (page_setup_p))) {
+	gtk_page_setup_set_top_margin(page_setup_p, Cgu::STANDARD_MARGINS, GTK_UNIT_MM);
+	gtk_page_setup_set_bottom_margin(page_setup_p, Cgu::STANDARD_MARGINS, GTK_UNIT_MM);
+	gtk_page_setup_set_left_margin(page_setup_p, Cgu::STANDARD_MARGINS, GTK_UNIT_MM);
+	gtk_page_setup_set_right_margin(page_setup_p, Cgu::STANDARD_MARGINS, GTK_UNIT_MM);
+      }
+      // The documentation is unclear whether we get a new GtkPageSetup
+      // object back if an existing GtkPageSetup object was passed to
+      // gtk_print_run_page_setup_dialog_async().  However this is safe
+      // even if we get the same object back again - see GobjHandle
+      // implementation
+      Cgu::TextPrintManager::page_setup_h = Cgu::GobjHandle<GtkPageSetup>(page_setup_p);
+    }
+    if (data) gtk_widget_set_sensitive(GTK_WIDGET(data), true);
+  }
 
-void CguTextPrintManagerCB::text_print_custom_widget_apply(GtkPrintOperation*, GtkWidget*,
-							   void* data) {
+  static GObject* create_custom_widget(GtkPrintOperation* print_operation_p,
+				       void* data) {
 
-  Cgu::TextPrintManager* instance_p = static_cast<Cgu::TextPrintManager*>(data);
+    return static_cast<Cgu::TextPrintManager*>(data)->create_custom_widget_impl(print_operation_p);
+  }
 
-  instance_p->font_size =
-    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(instance_p->font_size_spin_button_h.get()));
-  instance_p->font_family = gtk_entry_get_text(GTK_ENTRY(instance_p->font_entry_h.get()));
-  Cgu::TextPrintManager::strip(instance_p->font_family);
+  static void custom_widget_apply(void* data) {
 
-  Cgu::TextPrintManager::default_font_size = instance_p->font_size;
-  Cgu::TextPrintManager::default_font_family = instance_p->font_family;
-}
+    Cgu::TextPrintManager* instance_p = static_cast<Cgu::TextPrintManager*>(data);
 
-} // anonymous namespace
-#endif
+    instance_p->font_size =
+      gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(instance_p->font_size_spin_button_h.get()));
+    instance_p->font_family = gtk_entry_get_text(GTK_ENTRY(instance_p->font_entry_h.get()));
+    Cgu::TextPrintManager::strip(instance_p->font_family);
+
+    Cgu::TextPrintManager::default_font_size = instance_p->font_size;
+    Cgu::TextPrintManager::default_font_family = instance_p->font_family;
+  }
+};
+
+} // namespace Cgu
+
+// the GObject callback functions with both C linkage
+// specification and internal linkage
+extern "C" {
+  static void cgu_tpm_begin_print(GtkPrintOperation* print_operation_p,
+				  GtkPrintContext* context_p,
+				  void* data) {
+    Cgu::TextPrintManager::CB::begin_print(print_operation_p, context_p, data);
+  }
+
+  static void cgu_tpm_draw_page(GtkPrintOperation* print_operation_p,
+				GtkPrintContext* context_p,
+				gint page_nr,
+				void* data) {
+    Cgu::TextPrintManager::CB::draw_page(print_operation_p, context_p, page_nr, data);
+  }
+
+  static void cgu_tpm_done(GtkPrintOperation* print_operation_p,
+			   GtkPrintOperationResult result,
+			   void* data) {
+    Cgu::TextPrintManager::CB::done(print_operation_p, result,  data);
+  }
+
+  static void cgu_tpm_page_setup_done(GtkPageSetup* page_setup_p,
+				      void* data) {
+    Cgu::TextPrintManager::CB::page_setup_done(page_setup_p, data);
+  }
+
+  static GObject* cgu_tpm_create_custom_widget(GtkPrintOperation* print_operation_p,
+					       void* data) {
+    return Cgu::TextPrintManager::CB::create_custom_widget(print_operation_p, data);
+  }
+
+  static void cgu_tpm_custom_widget_apply(GtkPrintOperation*,
+					  GtkWidget*,
+					  void* data) {
+    Cgu::TextPrintManager::CB::custom_widget_apply(data);
+  }
+} // extern "C"
+
+#endif // DOXYGEN_PARSING
 
 namespace Cgu {
 
@@ -190,7 +233,7 @@ void TextPrintManager::page_setup(GtkWindow* parent) {
 
   if (parent) gtk_widget_set_sensitive(GTK_WIDGET(parent), false);
   gtk_print_run_page_setup_dialog_async(parent, page_setup_h, print_settings_h, 
-					CguTextPrintManagerCB::text_print_page_setup_done,
+					cgu_tpm_page_setup_done,
 					parent);
 }
 
@@ -288,15 +331,15 @@ void TextPrintManager::print_text() {
   gtk_print_operation_set_default_page_setup(print_operation_h, page_setup_h);
   
   g_signal_connect(G_OBJECT(print_operation_h.get()), "begin_print",
-		   G_CALLBACK(CguTextPrintManagerCB::text_print_begin_print), this);
+		   G_CALLBACK(cgu_tpm_begin_print), this);
   g_signal_connect(G_OBJECT(print_operation_h.get()), "draw_page",
-		   G_CALLBACK(CguTextPrintManagerCB::text_print_draw_page), this);
+		   G_CALLBACK(cgu_tpm_draw_page), this);
 #ifdef TEXT_PRINT_MANAGER_ALLOW_ASYNC
   g_signal_connect(G_OBJECT(print_operation_h.get()), "done",
-		   G_CALLBACK(CguTextPrintManagerCB::text_print_done), this);
+		   G_CALLBACK(cgu_tpm_done), this);
   gtk_print_operation_set_allow_async(print_operation_h, true);
   // regain ownership of ourselves (the print system will do the final
-  // unreference in the CguTextPrintManagerCB::text_print_done() callback)
+  // unreference in the TextPrintManagerCB::done() callback)
   ref();
 #endif
 
@@ -324,9 +367,9 @@ void TextPrintManager::print_text() {
   }
   else {
     g_signal_connect(G_OBJECT(print_operation_h.get()), "create_custom_widget",
-		     G_CALLBACK(CguTextPrintManagerCB::text_print_create_custom_widget), this);
+		     G_CALLBACK(cgu_tpm_create_custom_widget), this);
     g_signal_connect(G_OBJECT(print_operation_h.get()), "custom_widget_apply",
-		     G_CALLBACK(CguTextPrintManagerCB::text_print_custom_widget_apply), this);
+		     G_CALLBACK(cgu_tpm_custom_widget_apply), this);
     result = gtk_print_operation_run(print_operation_h,
 				     GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
 				     parent_p, &error_p);
