@@ -89,8 +89,13 @@ Logger::Logger(const char* default_dir, GtkWindow* parent_p_):
 		<< gettext("Beginning fax log: ") << date_desc
 		<< " *****\n" << std::endl;
 	// set a timeout to flush every one minute
+#if GLIB_CHECK_VERSION(2,14,0)
+	timer_tag = start_timeout_seconds(LOGFILE_TIMER_INTERVAL/1000,
+					  Callback::make(*this, &Logger::logfile_timer_cb));
+#else
 	timer_tag = start_timeout(LOGFILE_TIMER_INTERVAL,
 				  Callback::make(*this, &Logger::logfile_timer_cb));
+#endif
       }
 
       // save the logfile name in the local version of logfile_name 
@@ -239,8 +244,13 @@ void Logger::reset_logfile(void) {
 		<< gettext("Beginning fax log: ") << date_desc
 		<< " *****\n" << std::endl;
 	// set a timeout to flush every one minute
+#if GLIB_CHECK_VERSION(2,14,0)
+	timer_tag = start_timeout_seconds(LOGFILE_TIMER_INTERVAL/1000,
+					  Callback::make(*this, &Logger::logfile_timer_cb));
+#else
 	timer_tag = start_timeout(LOGFILE_TIMER_INTERVAL,
 				  Callback::make(*this, &Logger::logfile_timer_cb));
+#endif
       }
       logfile_count = 0;
       // save the logfile name in the local version of logfile_name 
@@ -358,27 +368,32 @@ void Logger::logfile_timer_cb(bool&) {
   logfile.flush();
 }
 
-namespace { // callbacks for internal use only
-
-void LogViewDialogCB::log_view_button_clicked(GtkWidget*, void* data) {
+void LogViewDialogCB::log_view_dialog_button_clicked(GtkWidget*, void* data) {
   static_cast<LogViewDialog*>(data)->close();
 }
 
-gboolean LogViewDialogCB::log_view_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
+gboolean LogViewDialogCB::log_view_dialog_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
   LogViewDialog* instance_p = static_cast<LogViewDialog*>(data);
   int keycode = event_p->keyval;
   
+#if GTK_CHECK_VERSION(2,91,0)
+  if (keycode == GDK_KEY_Home || keycode == GDK_KEY_End
+      || keycode == GDK_KEY_Up || keycode == GDK_KEY_Down
+      || keycode == GDK_KEY_Page_Up || keycode == GDK_KEY_Page_Down) {
+    gtk_widget_event(GTK_WIDGET(instance_p->text_view_p), (GdkEvent*)event_p);
+    return true;    // stop processing here
+  }
+#else
   if (keycode == GDK_Home || keycode == GDK_End
       || keycode == GDK_Up || keycode == GDK_Down
       || keycode == GDK_Page_Up || keycode == GDK_Page_Down) {
     gtk_widget_event(GTK_WIDGET(instance_p->text_view_p), (GdkEvent*)event_p);
     return true;    // stop processing here
   }
+#endif
   return false;     // pass on the key event
 }
 
-} // anonymous namespace
-
 LogViewDialog::LogViewDialog(const int standard_size,
 			     const std::string& logfile_name,
 			     GtkWindow* parent_p):
@@ -442,9 +457,9 @@ LogViewDialog::LogViewDialog(const int standard_size,
   }
 
   g_signal_connect(G_OBJECT(close_button_p), "clicked",
-		   G_CALLBACK(LogViewDialogCB::log_view_button_clicked), this);
+		   G_CALLBACK(LogViewDialogCB::log_view_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(get_win()), "key_press_event",
-		   G_CALLBACK(LogViewDialogCB::log_view_key_press_event), this);
+		   G_CALLBACK(LogViewDialogCB::log_view_dialog_key_press_event), this);
 
   gtk_window_set_default_size(get_win(), standard_size * 25, standard_size * 16);
   
