@@ -49,7 +49,11 @@ COPYING distributed with the source files.
 
 #ifdef HAVE_X11_XLIB_H
 #include <X11/Xlib.h>
-#include <gdk/gdkx.h>  // for GDK_DISPLAY()
+#include <gdk/gdkx.h>  // for GDK_DISPLAY_XDISPLAY()
+#endif
+
+#ifdef HAVE_X11_XKBLIB_H
+#include <X11/XKBlib.h>  // for beep
 #endif
 
 #include <c++-gtk-utils/mutex.h>
@@ -84,7 +88,7 @@ void clean_up_receive_dir(void);
 bool is_ascii(const std::string&);
 bool get_prog_parm(const char*, std::string&, std::string&, std::string(*)(const std::string&));
 extern "C" void atexit_cleanup(void);
-extern "C" gboolean present_prog(void*, const char**);
+gboolean present_prog(void*, const char**);
 
 inline bool get_prog_parm(const char* name, std::string& line, std::string& result) {
   return get_prog_parm(name, line, result, Utf8::locale_to_utf8);
@@ -864,7 +868,7 @@ void get_fonts(void) {
   // this will get a suitable fixed font for GplDialog and HelpDialog to use with Pango
   const int MAX_FONTS = 10000;
   int num_fonts;
-  char** fonts = XListFonts(GDK_DISPLAY(), "-*", MAX_FONTS, &num_fonts);
+  char** fonts = XListFonts(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), "-*", MAX_FONTS, &num_fonts);
 
   if (fonts) {
     int count;
@@ -929,8 +933,10 @@ bool get_prog_parm(const char* name, std::string& line, std::string& result,
 }
 
 void beep(void) {
-#ifdef HAVE_X11_XLIB_H
-  XBell(GDK_DISPLAY(), 0);
+#if defined(HAVE_X11_XKBLIB_H) && defined(HAVE_X11_XLIB_H)
+  XkbBell(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), None, 0, None);
+#elif defined(HAVE_X11_XLIB_H)
+  XBell(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), 0);
 #else
   gdk_beep();
 #endif
@@ -943,12 +949,15 @@ void get_window_icon(void) {
   GtkIconTheme* icon_theme_p = gtk_icon_theme_get_default();
 
   IconInfoScopedHandle icon_info_h(gtk_icon_theme_lookup_icon(icon_theme_p,
-							      "stock_send-fax",
+							      "efax-gtk",
 							      24, GtkIconLookupFlags(0)));
   if (icon_info_h.get()) {
       
     const gchar* icon_path_p = gtk_icon_info_get_filename(icon_info_h.get());
     if (icon_path_p) {
+      // for debugging
+      //std::cout << "Got efax-gtk icon" << std::endl;
+
       GError* error_p = 0;
       prog_config.window_icon_h =
 	GobjHandle<GdkPixbuf>(gdk_pixbuf_new_from_file(icon_path_p, &error_p));
