@@ -21,9 +21,12 @@ COPYING distributed with the source files.
 #include <istream>
 #include <cstdlib>
 #include <cstring>
+#include <cmath>
 
 #include <gdk/gdkkeysyms.h> // the key codes are here
 #include <pango/pango-layout.h>
+#include <pango/pango-font.h>
+#include <cairo.h>
 
 #include "mainwindow.h"
 #include "dialogs.h"
@@ -33,6 +36,9 @@ COPYING distributed with the source files.
 #include "socket_notify.h"
 #include "menu_icons.h"
 #include "utils/icon_info_handle.h"
+#if !(GTK_CHECK_VERSION(2,91,0))
+#include "utils/cairo_handle.h"
+#endif
 
 #include <c++-gtk-utils/callback.h>
 #include <c++-gtk-utils/shared_handle.h>
@@ -53,16 +59,18 @@ COPYING distributed with the source files.
 #include <libintl.h>
 #endif
 
-#define MAINWIN_DRAWING_AREA_WIDTH 8
-#define MAINWIN_DRAWING_AREA_HEIGHT 16
-
-// there can be certain circumstances where PIP_BUF is not
+// there can be certain circumstances where PIPE_BUF is not
 // defined in <limits.h>.  If so, just define the minimum
 // required by POSIX
 #ifndef PIPE_BUF
 #define PIPE_BUF 512
 #endif
 
+// and make sure M_PI is defined
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
+
 extern "C" void close_signalhandler(int);
 PipeFifo sig_pipe(PipeFifo::non_block);
 
@@ -72,8 +80,6 @@ bool MainWindow::connected_to_stderr = false;
 PipeFifo MainWindow::error_pipe(PipeFifo::non_block);
 
 
-namespace { // callbacks for internal use only
-
 void MainWindowCB::mainwin_button_clicked(GtkWidget* widget_p, void* data) {
   MainWindow* instance_p = static_cast<MainWindow*>(data);
 
@@ -175,10 +181,17 @@ void MainWindowCB::mainwin_menuitem_activated(GtkMenuItem* item_p, void* data) {
 gboolean MainWindowCB::mainwin_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
   MainWindow* instance_p = static_cast<MainWindow*>(data);
   
+#if GTK_CHECK_VERSION(2,91,0)
+  if (event_p->keyval == GDK_KEY_F1) {
+    instance_p->helpfile_impl();
+    return true; // processing stops here
+  }
+#else
   if (event_p->keyval == GDK_F1) {
     instance_p->helpfile_impl();
     return true; // processing stops here
   }
+#endif
   return false;  // carry on processing the key event
 }
 
@@ -204,18 +217,23 @@ gboolean MainWindowCB::mainwin_window_state_event(GtkWidget*, GdkEventWindowStat
   return false;  // carry on processing the window state event
 }
 
-void MainWindowCB::mainwin_style_set(GtkWidget*, GtkStyle*, void* data) {
-  static_cast<MainWindow*>(data)->style_set_impl();
+void MainWindowCB::mainwin_style_set(GtkWidget*, GtkStyle* prev_style_p, void* data) {
+  static_cast<MainWindow*>(data)->style_set_impl(prev_style_p);
 }
 
+#if GTK_CHECK_VERSION(2,91,0)
+gboolean MainWindowCB::mainwin_drawing_area_draw(GtkWidget*,
+						 cairo_t* cairo_p,
+						 void* data) {
+  return static_cast<MainWindow*>(data)->draw_fax_from_socket_notifier(cairo_p);
+}
+#else
 gboolean MainWindowCB::mainwin_drawing_area_expose_event(GtkWidget*,
-							 GdkEventExpose*,
+							 GdkEventExpose* event_p,
 							 void* data) {
-  return static_cast<MainWindow*>(data)->draw_fax_from_socket_notifier();
+  return static_cast<MainWindow*>(data)->draw_fax_from_socket_notifier(event_p);
 }
-
-} // anonymous namespace
-
+#endif
 
 MainWindow::MainWindow(const std::string& messages,bool start_hidden,
 		       bool start_in_standby, const char* filename):
@@ -256,8 +274,6 @@ MainWindow::MainWindow(const std::string& messages,bool start_hidden,
   // exit status in EfaxController::sendfax_thread() or
   // EfaxController::receive_thread, which call waitpid()
 
-  get_longest_button_text();
-
   GtkWidget* drawing_area_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
   drawing_area_p = gtk_drawing_area_new();
   gtk_widget_set_size_request(drawing_area_p, standard_size, standard_size);
@@ -713,9 +729,15 @@ MainWindow::MainWindow(const std::string& messages,bool start_hidden,
   gtk_container_set_focus_chain(GTK_CONTAINER(win_table_p), focus_chain_p);
   g_list_free(focus_chain_p);
 
+#if GTK_CHECK_VERSION(2,91,0)
+  g_signal_connect(G_OBJECT(drawing_area_p), "draw",
+		   G_CALLBACK(MainWindowCB::mainwin_drawing_area_draw),
+		   this);
+#else
   g_signal_connect(G_OBJECT(drawing_area_p), "expose_event",
 		   G_CALLBACK(MainWindowCB::mainwin_drawing_area_expose_event),
 		   this);
+#endif
 
   efax_controller.ready_to_quit_notify.connect(Callback::make(*this, &MainWindow::quit_cb));
 
@@ -1558,7 +1580,7 @@ void MainWindow::translations_impl(void) {
   message += gettext("Hebrew - Assaf Gillat\n");
   message += gettext("Greek - Hellenic Linux Users Group\n");
   message += gettext("Albanian - Besnik Bleta\n");
-  message += gettext("Hungarian - Gergely Szakats\n");
+  message += gettext("Hungarian - Gergely Szakats, Laszlo Csordas\n");
   message += gettext("Simplified Chinese - Kite Lau\n");
   message += gettext("German - Steffen Wagner\n");
   message += gettext("Swedish - Daniel Nylander\n");
@@ -1572,7 +1594,7 @@ void MainWindow::translations_impl(void) {
   // will  delete its own memory when closed
 }
 
-void MainWindow::get_longest_button_text(void) {
+std::pair<int, int> MainWindow::get_max_button_text_extents(void) {
   std::vector<std::string> text_vec;
   text_vec.push_back(gettext("Single file"));
   text_vec.push_back(gettext("Multiple files"));
@@ -1584,29 +1606,61 @@ void MainWindow::get_longest_button_text(void) {
   text_vec.push_back(gettext("Standby"));
   text_vec.push_back(gettext("Stop"));
 
-  std::vector<std::string>::const_iterator temp_iter;
-  std::vector<std::string>::const_iterator max_width_iter;
+  std::vector<std::string>::const_iterator iter;
   int width;
   int height;
   int max_width;
+  int max_height;
 
   // create a button to work on - the GobjHandle object will claim ownership of the button
+  GtkStyle* style_p = gtk_widget_get_style(GTK_WIDGET(get_win()));
   GobjHandle<GtkWidget> button_h(gtk_button_new());
+  gtk_widget_set_style(button_h, style_p);
 
-  for (temp_iter = text_vec.begin(), max_width_iter = text_vec.begin(),
-	 width = 0, height = 0, max_width = 0;
-       temp_iter != text_vec.end(); ++temp_iter) {
+  for (iter = text_vec.begin(), width = 0, height = 0, max_width = 0, max_height = 0;
+       iter != text_vec.end();
+       ++iter) {
 
-    GobjHandle<PangoLayout> pango_layout_h(gtk_widget_create_pango_layout(button_h,
-									 temp_iter->c_str()));
-    pango_layout_get_pixel_size(pango_layout_h, &width, &height);
+    GobjHandle<PangoLayout> layout_h(gtk_widget_create_pango_layout(button_h,
+								    iter->c_str()));
+    pango_layout_get_pixel_size(layout_h, &width, &height);
 
     if (width > max_width) {
       max_width = width;
-      max_width_iter = temp_iter;
+    }
+    if (height > max_height) {
+      max_height = height;
     }
   }
-  max_text = *max_width_iter;
+  return std::pair<int, int>(max_width, max_height);
+}
+
+int MainWindow::get_max_status_text_height(void) {
+  std::vector<std::string> text_vec(efax_controller.get_state_messages());
+  text_vec.push_back(gettext("Press F1 for help"));
+  std::vector<std::string>::const_iterator iter;
+  int width;
+  int height;
+  int max_height;
+
+  // create a label to work on - the GobjHandle object will claim ownership of the label
+  GtkStyle* style_p = gtk_widget_get_style(GTK_WIDGET(get_win()));
+  GobjHandle<GtkWidget> label_h(gtk_label_new(0));
+  gtk_widget_set_style(label_h, style_p);
+
+  for (iter = text_vec.begin(), width = 0, height = 0, max_height = 0;
+       iter != text_vec.end();
+       ++iter) {
+
+    GobjHandle<PangoLayout> layout_h(gtk_widget_create_pango_layout(label_h,
+								    iter->c_str()));
+    pango_layout_get_pixel_size(layout_h, &width, &height);
+
+    if (height > max_height) {
+      max_height = height;
+    }
+  }
+  return max_height;
 }
 
 void MainWindow::tray_icon_activated_cb(void) {
@@ -1666,46 +1720,51 @@ void MainWindow::tray_icon_menu_cb(int item) {
   }
 }
 
-void MainWindow::style_set_impl(void) {
+void MainWindow::style_set_impl(GtkStyle* prev_style_p) {
 
-  // the main task is to set the buttons in MainWindow
-  int width = 0;
-  int height = 0;
-  // create a button to work on - the GobjHandle object will claim ownership of the button
-  GobjHandle<GtkWidget> button_h(gtk_button_new());
+  GtkStyle* new_style_p = gtk_widget_get_style(GTK_WIDGET(get_win()));
+  // we only need to proceed further if the font has changed
+  if (!prev_style_p || !pango_font_description_equal(prev_style_p->font_desc,
+						     new_style_p->font_desc)) {
 
-  GtkStyle* style_p = gtk_widget_get_style(GTK_WIDGET(get_win()));
-  gtk_widget_set_style(button_h, style_p);
+    std::pair<int, int> extents = get_max_button_text_extents();
 
-  GobjHandle<PangoLayout> pango_layout_h(gtk_widget_create_pango_layout(button_h,
-							 max_text.c_str()));
-  pango_layout_get_pixel_size(pango_layout_h, &width, &height);
+    // add padding
+    extents.first += 18;  // width
+    extents.second += 12; // height
 
-  // add padding
-  width += 18;
-  height += 12;
+    // have some sensible minimum width and height if a very small font chosen
+    const int button_min_width = standard_size * 4;
+    const int button_min_height = 30;
+    if (extents.first < button_min_width) extents.first = button_min_width;
+    if (extents.second < button_min_height) extents.second = button_min_height;
 
-  // have some sensible minimum width and height if a very small font chosen
-  const int min_width = standard_size * 4;
-  const int min_height = 30;
-  if (width < min_width) width = min_width;
-  if (height < min_height) height = min_height;
+    gtk_widget_set_size_request(single_file_button_p, extents.first, extents.second);
+    gtk_widget_set_size_request(multiple_file_button_p, extents.first, extents.second);
+    gtk_widget_set_size_request(socket_list_button_p, extents.first, extents.second);
+    gtk_widget_set_size_request(number_button_p, extents.first, extents.second);
+    gtk_widget_set_size_request(send_button_p, extents.first, extents.second);
+    gtk_widget_set_size_request(receive_answer_button_p, extents.first, extents.second);
+    gtk_widget_set_size_request(receive_takeover_button_p, extents.first, extents.second);
+    gtk_widget_set_size_request(receive_standby_button_p, extents.first, extents.second);
+    gtk_widget_set_size_request(stop_button_p, extents.first, extents.second);
 
-  gtk_widget_set_size_request(single_file_button_p, width, height);
-  gtk_widget_set_size_request(multiple_file_button_p, width, height);
-  gtk_widget_set_size_request(socket_list_button_p, width, height);
-  gtk_widget_set_size_request(number_button_p, width, height);
-  gtk_widget_set_size_request(send_button_p, width, height);
-  gtk_widget_set_size_request(receive_answer_button_p, width, height);
-  gtk_widget_set_size_request(receive_takeover_button_p, width, height);
-  gtk_widget_set_size_request(receive_standby_button_p, width, height);
-  gtk_widget_set_size_request(stop_button_p, width, height);
+    int status_height = get_max_status_text_height();
 
-  // now set the status line style
-  status_line.set_status_line(height - 6);
+    // add padding
+    status_height += 2;
+
+    // set_status_line() provides a minimum height if a very small font chosen
+    status_line.set_status_line(status_height);
+  }
 }
 
-bool MainWindow::draw_fax_from_socket_notifier(void) {
+#if GTK_CHECK_VERSION(2,91,0)
+bool MainWindow::draw_fax_from_socket_notifier(cairo_t* cairo_p)
+#else
+bool MainWindow::draw_fax_from_socket_notifier(GdkEventExpose* event_p)
+#endif
+                                                                        {
 
 /* This method displays the "red circle" indicator which shows a fax
    file received from the socket is awaiting sending.  It is is called
@@ -1753,104 +1812,56 @@ bool MainWindow::draw_fax_from_socket_notifier(void) {
      from the socket)
 */
 
-#if GTK_CHECK_VERSION(2,20,0)
+#if GTK_CHECK_VERSION(2,91,0)
   if (gtk_widget_get_realized(drawing_area_p)) { // if we started hidden in the system tray
                                                  // the drawing area may not yet be realised
-
-    GdkDrawable* drawable_p = GDK_DRAWABLE(gtk_widget_get_window(drawing_area_p));
-
     if (!socket_server.get_filenames().first->empty()) {
-
-      if (!red_gc_h.get()) {
-      
-	GdkColor red;
-	red.red = static_cast<guint16>(0.7 * 65535.0);
-	red.green = 0;
-	red.blue = 0;
-
-	GdkColormap* colormap_p = gtk_widget_get_default_colormap();
-	if (!gdk_colormap_alloc_color(colormap_p, &red, false, true)) {
-	  write_error("Error allocating colour in colourmap in "
-		      "MainWindow::draw_fax_from_socket_notifier()\n");
-	}
-	else {
-	  red_gc_h = GobjHandle<GdkGC>(gdk_gc_new(drawable_p));
-	  gdk_gc_set_foreground(red_gc_h, &red);
-	}
-      }
-
-      if (red_gc_h.get()) {
-
-	GdkGC* white_gc_p = gtk_widget_get_style(drawing_area_p)->white_gc;
-
-	gdk_draw_arc(drawable_p, white_gc_p,
-		     true, 6, 6,
-		     standard_size - 12, standard_size - 12,
-		     0, 64 * 360);
-	
-	gdk_draw_arc(drawable_p, red_gc_h,
-		     true, 8, 8,
-		     standard_size - 16, standard_size - 16,
-		     0, 64 * 360);
-      }
+      cairo_arc(cairo_p, 
+		gtk_widget_get_allocated_width(drawing_area_p)/2,
+		gtk_widget_get_allocated_height(drawing_area_p)/2,
+		5,
+		0, 2 * M_PI);
+      cairo_set_source_rgb(cairo_p, 0.7, 0.0, 0.0);
+      cairo_fill_preserve(cairo_p);
+      cairo_set_source_rgb(cairo_p, 1.0, 1.0, 1.0);
+      cairo_set_line_width(cairo_p, 2);
+      cairo_stroke(cairo_p);
     }
     else {
-      
-      GdkGC* background_gc_p =
-	gtk_widget_get_style(drawing_area_p)->bg_gc[gtk_widget_get_state(drawing_area_p)];
-      gdk_draw_rectangle(drawable_p, background_gc_p,
-			 true, 0, 0,
-			 standard_size, standard_size);
-    } 
+      gdk_cairo_set_source_color(cairo_p,
+				 &gtk_widget_get_style(drawing_area_p)->bg[gtk_widget_get_state(drawing_area_p)]);
+      cairo_paint(cairo_p);
+    }
   }
 #else
   if (GTK_WIDGET_REALIZED(drawing_area_p)) { // if we started hidden in the system tray
                                              // the drawing area may not yet be realised
 
-    GdkDrawable* drawable_p = GDK_DRAWABLE(drawing_area_p->window);
+    // provide the cairo context incorporating the destination surface
+    CairoContextScopedHandle cairo_h(gdk_cairo_create(GDK_DRAWABLE(drawing_area_p->window)));
+
+    // clip any redrawing to the area of the expose event
+    cairo_rectangle(cairo_h,
+		    event_p->area.x, event_p->area.y,
+		    event_p->area.width, event_p->area.height);
+    cairo_clip(cairo_h);
 
     if (!socket_server.get_filenames().first->empty()) {
-
-      if (!red_gc_h.get()) {
-      
-	GdkColor red;
-	red.red = static_cast<guint16>(0.7 * 65535.0);
-	red.green = 0;
-	red.blue = 0;
-
-	GdkColormap* colormap_p = gtk_widget_get_default_colormap();
-	if (!gdk_colormap_alloc_color(colormap_p, &red, false, true)) {
-	  write_error("Error allocating colour in colourmap in "
-		      "MainWindow::draw_fax_from_socket_notifier()\n");
-	}
-	else {
-	  red_gc_h = GobjHandle<GdkGC>(gdk_gc_new(drawable_p));
-	  gdk_gc_set_foreground(red_gc_h, &red);
-	}
-      }
-
-      if (red_gc_h.get()) {
-
-	GdkGC* white_gc_p = gtk_widget_get_style(drawing_area_p)->white_gc;
-
-	gdk_draw_arc(drawable_p, white_gc_p,
-		     true, 6, 6,
-		     standard_size - 12, standard_size - 12,
-		     0, 64 * 360);
-	
-	gdk_draw_arc(drawable_p, red_gc_h,
-		     true, 8, 8,
-		     standard_size - 16, standard_size - 16,
-		     0, 64 * 360);
-      }
+      cairo_arc(cairo_h, 
+		(drawing_area_p->allocation.width)/2,
+		(drawing_area_p->allocation.height)/2,
+		5,
+		0, 2 * M_PI);
+      cairo_set_source_rgb(cairo_h, 0.7, 0.0, 0.0);
+      cairo_fill_preserve(cairo_h);
+      cairo_set_source_rgb(cairo_h, 1.0, 1.0, 1.0);
+      cairo_set_line_width(cairo_h, 2);
+      cairo_stroke(cairo_h);
     }
     else {
-      
-      GdkGC* background_gc_p =
-	gtk_widget_get_style(drawing_area_p)->bg_gc[GTK_WIDGET_STATE(drawing_area_p)];
-      gdk_draw_rectangle(drawable_p, background_gc_p,
-			 true, 0, 0,
-			 standard_size, standard_size);
+      gdk_cairo_set_source_color(cairo_h,
+				 &gtk_widget_get_style(drawing_area_p)->bg[GTK_WIDGET_STATE(drawing_area_p)]);
+      cairo_paint(cairo_h);
     } 
   }
 #endif
@@ -1931,6 +1942,11 @@ MessageText::MessageText(GtkWindow* win_p):
   // GtkTextTags are not owned by a GTK+ container when passed to it so use a GobjHandle
   red_tag_h = GobjHandle<GtkTextTag>(gtk_text_tag_new(0));
 
+#if GTK_CHECK_VERSION(2,91,0)
+  g_object_set(red_tag_h.get(),
+	       "foreground-gdk", &red,
+	       static_cast<void*>(0));
+#else
   GdkColormap* colormap_p = gtk_widget_get_default_colormap();
   if (!gdk_colormap_alloc_color(colormap_p, &red, false, true)) {
     write_error("Error allocating colour in colourmap in "
@@ -1941,6 +1957,7 @@ MessageText::MessageText(GtkWindow* win_p):
 		 "foreground-gdk", &red,
 		 static_cast<void*>(0));
   }
+#endif
 
   GtkTextBuffer* buffer_p = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view_p));
   gtk_text_tag_table_add(gtk_text_buffer_get_tag_table(buffer_p), red_tag_h);
@@ -1990,10 +2007,17 @@ void MessageText::write_black_cb(const char* message) {
 
   bool scrolling = false;
   GtkAdjustment* adj_p = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(get_main_widget()));
+#if GTK_CHECK_VERSION(2,14,0)
   if (gtk_adjustment_get_value(adj_p)
       >= (gtk_adjustment_get_upper(adj_p) - gtk_adjustment_get_page_size(adj_p) - 1e-12)) {
     scrolling = true;
   }
+#else
+  if (gtk_adjustment_get_value(adj_p)
+      >= (adj_p->upper - adj_p->page_size - 1e-12)) {
+    scrolling = true;
+  }
+#endif
   
   GtkTextBuffer* buffer_p = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view_p));
   GtkTextIter end_iter;
@@ -2028,10 +2052,17 @@ void MessageText::write_red_cb(const char* message) {
 
   bool scrolling = false;
   GtkAdjustment* adj_p = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(get_main_widget()));
+#if GTK_CHECK_VERSION(2,14,0)
   if (gtk_adjustment_get_value(adj_p)
       >= (gtk_adjustment_get_upper(adj_p) - gtk_adjustment_get_page_size(adj_p) - 1e-12)) {
     scrolling = true;
   }
+#else
+  if (gtk_adjustment_get_value(adj_p)
+      >= (adj_p->upper - adj_p->page_size - 1e-12)) {
+    scrolling = true;
+  }
+#endif
   
   GtkTextBuffer* buffer_p = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view_p));
   GtkTextIter end_iter;
@@ -2089,8 +2120,8 @@ StatusLine::StatusLine(const int size):
 void StatusLine::set_status_line(int new_height) {
 
   // set the height
-  if (new_height < standard_size + 2) new_height = standard_size + 2;
-  gtk_widget_set_size_request(get_main_widget(), -1, new_height);
+  if (new_height < standard_size) new_height = standard_size;
+  gtk_widget_set_size_request(status_label_p, -1, new_height);
 
   // make the status label display in red
   GdkColor red;
