@@ -19,8 +19,8 @@ COPYING distributed with the source files.
 #include <list>
 #include <utility>
 
-#include <gtk/gtk.h>
 #include <gdk/gdk.h>
+#include <gtk/gtk.h>
 #include <glib.h>
 
 #include "efax_controller.h"
@@ -72,8 +72,6 @@ public:
   StatusLine(const int);
 };
 
-namespace { // we put the functions in anonymous namespace in mainwindow.cpp
-            // so they are not exported at link time
 namespace MainWindowCB {
   extern "C" {
     void mainwin_button_clicked(GtkWidget*, void*);
@@ -83,10 +81,13 @@ namespace MainWindowCB {
     gboolean mainwin_window_state_event(GtkWidget*, GdkEventWindowState*, void*);
     void mainwin_style_set(GtkWidget*, GtkStyle*, void*);
     gboolean mainwin_timer_event(void*);
+#if GTK_CHECK_VERSION(2,91,0)
+    gboolean mainwin_drawing_area_draw(GtkWidget*, cairo_t*, void*);
+#else
     gboolean mainwin_drawing_area_expose_event(GtkWidget*, GdkEventExpose*, void*);
+#endif
   }
 }
-}
 
 class MainWindow: public WinBase {
 
@@ -134,7 +135,6 @@ class MainWindow: public WinBase {
   GtkMenuItem* translations_item_p;
   GtkMenuItem* help_item_p;
 
-  std::string max_text;
   MessageText text_window;
   StatusLine status_line;
   EfaxController efax_controller;
@@ -152,8 +152,6 @@ class MainWindow: public WinBase {
 
   Utf8::Reassembler error_pipe_reassembler;
 
-  GobjHandle<GdkGC> red_gc_h;
-
   // these are called by handlers of GTK+ signals
   void close_impl(void);
   void get_file_impl(void);
@@ -169,7 +167,7 @@ class MainWindow: public WinBase {
   void helpfile_impl(void);
   void set_file_items_sensitive_impl(void);
   void set_socket_items_sensitive_impl(void);
-  void style_set_impl(void);
+  void style_set_impl(GtkStyle*);
 
   // these are connected to Emitter or Notifier objects, or connected
   // to the callback of a WatchSource (iowatch) object, and might also be
@@ -191,11 +189,17 @@ class MainWindow: public WinBase {
   void start_hidden_check_cb(bool&);
 
   void fax_to_send_dialog(const std::pair<std::string, unsigned int>&);
-  void get_longest_button_text(void);
-  bool draw_fax_from_socket_notifier(void);
+  std::pair<int, int> get_max_button_text_extents(void);
+  int get_max_status_text_height(void);
   void get_window_settings(void);
   void save_window_settings(void);
   void strip(std::string&);
+
+#if GTK_CHECK_VERSION(2,91,0)
+  bool draw_fax_from_socket_notifier(cairo_t*);
+#else
+  bool draw_fax_from_socket_notifier(GdkEventExpose*);
+#endif
 protected:
   virtual void on_delete_event(void);
 public:
@@ -207,8 +211,13 @@ public:
   friend gboolean MainWindowCB::mainwin_window_state_event(GtkWidget*,
 							   GdkEventWindowState*, void*);
   friend void MainWindowCB::mainwin_style_set(GtkWidget*, GtkStyle*, void*);
+#if GTK_CHECK_VERSION(2,91,0)
+  friend gboolean MainWindowCB::mainwin_drawing_area_draw(GtkWidget*,
+							  cairo_t*, void*);
+#else
   friend gboolean MainWindowCB::mainwin_drawing_area_expose_event(GtkWidget*,
 								  GdkEventExpose*, void*);
+#endif
 
   friend ssize_t write_error(const char*);
   friend int connect_to_stderr(void);
