@@ -184,7 +184,7 @@ using namespace Cgu;
 5. See also the comments about thread sefety in the function
    write_error() in mainwindow.cpp.
 
-6. this program used libsigc++ prior to version 3.2.0.
+6. This program used libsigc++ prior to version 3.2.0.
    sigc::trackable in libsigc++ is not thread safe - amongst other
    things, a class inheriting from sigc::trackable will, via that
    inheritance, have a std::list object keeping track of slots
@@ -202,7 +202,7 @@ using namespace Cgu;
        disconnect slots with respect to the signal object, and only
        that thread should emit() or call operator()() on it.
 
-     - Unless special additional synchronisaton is employed, regard
+     - Unless special additional synchronisation is employed, regard
        any sigc::connection object as owned by the thread which
        created the slot and called the method which provided the
        sigc::connection object.  Only that thread should call
@@ -218,10 +218,7 @@ using namespace Cgu;
        non-static methods (ie create slots with sigc::mem_fun()).
        Regard the first thread to create such a slot as owning the
        relevant object for the purpose of creating further slots
-       referencing any of its methods.  (It is safe however to make
-       objects of the opaque callback classes in the
-       src/utils/callback.h file in any thread, as these are thread
-       safe and do not have trackability.)
+       referencing any of its methods.
 
    From version 3.2.0, this program now uses the thread-safe Releaser
    and EmitterArg<> classes in the c++-gtk-utils library.
