@@ -241,14 +241,10 @@ PageMessages::PageMessages(void): SettingsMessagesBase(page + 1) {
 }
 
 
-namespace { // callback for internal use only
-
-void SettingsHelpDialogCB::settings_help_button_clicked(GtkWidget*, void* data) {
+void SettingsHelpDialogCB::settings_help_dialog_button_clicked(GtkWidget*, void* data) {
   static_cast<SettingsHelpDialog*>(data)->close();
 }
 
-} // anonymous namespace
-
 
 SettingsHelpDialog::SettingsHelpDialog(const int standard_size, const std::string& text,
 				       const std::string& caption, GtkWindow* parent_p):
@@ -277,7 +273,7 @@ SettingsHelpDialog::SettingsHelpDialog(const int standard_size, const std::strin
 		   standard_size/2, standard_size/4);
   
   g_signal_connect(G_OBJECT(close_button_p), "clicked",
-		   G_CALLBACK(SettingsHelpDialogCB::settings_help_button_clicked), this);
+		   G_CALLBACK(SettingsHelpDialogCB::settings_help_dialog_button_clicked), this);
 
 #if GTK_CHECK_VERSION(2,20,0)
   gtk_widget_set_can_default(close_button_p, true);
