@@ -121,18 +121,15 @@ public:
 };
 
 
-namespace { // we put the function in anonymous namespace in settings_help.cpp
-            // so it is not exported at link time
 namespace SettingsHelpDialogCB {
-  extern "C"  void settings_help_button_clicked(GtkWidget*, void*);
-}
+  extern "C"  void settings_help_dialog_button_clicked(GtkWidget*, void*);
 }
 
 class SettingsHelpDialog: public WinBase {
 
   GtkWidget* close_button_p;
 public:
-  friend void SettingsHelpDialogCB::settings_help_button_clicked(GtkWidget*, void*);
+  friend void SettingsHelpDialogCB::settings_help_dialog_button_clicked(GtkWidget*, void*);
 
   SettingsHelpDialog(const int standard_size, const std::string& text,
 		     const std::string& caption, GtkWindow* parent_p);
