@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2006 and 2009 Chris Vine
+/* Copyright (C) 2001 to 2006, 2009 and 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -34,75 +34,94 @@ COPYING distributed with the source files.
 
 int SocketListDialog::is_socket_list = 0;
 
+namespace {
 namespace ModelColumns {
   enum {fax_label, fax_filename, cols_num};
-}
-
-namespace { // callbacks for internal use only
-
-void SocketListDialogCB::socket_list_button_clicked(GtkWidget* widget_p, void* data) {
-  SocketListDialog* instance_p = static_cast<SocketListDialog*>(data);
-
-  if (widget_p == instance_p->send_button_p) {
-    instance_p->send_fax_impl();
-  }
-  else if (widget_p == instance_p->view_button_p) {
-    instance_p->view_file();
-  }
-  else if (widget_p == instance_p->remove_button_p) {
-    instance_p->remove_file_prompt();
-  }
-  else if (widget_p == instance_p->close_button_p) {
-    instance_p->close();
-  }
-  else {
-    write_error("Callback error in SocketListDialogCB::socket_list_button_clicked()\n");
-    instance_p->close();
-  }
-}
-
-void SocketListDialogCB::socket_list_set_buttons(GtkTreeSelection*, void* data) {
-  SocketListDialog* instance_p = static_cast<SocketListDialog*>(data);
-
-  // see if anything is selected
-  GtkTreeSelection* selection_p = gtk_tree_view_get_selection(instance_p->tree_view_p);
-  if (gtk_tree_selection_get_selected(selection_p, 0, 0)) {
-    gtk_widget_set_sensitive(instance_p->view_button_p, true);
-    gtk_widget_set_sensitive(instance_p->remove_button_p, true);
-    gtk_widget_set_sensitive(instance_p->send_button_p, true);
-    gtk_button_set_relief(GTK_BUTTON(instance_p->view_button_p), GTK_RELIEF_NORMAL);
-    gtk_button_set_relief(GTK_BUTTON(instance_p->remove_button_p), GTK_RELIEF_NORMAL);
-    gtk_button_set_relief(GTK_BUTTON(instance_p->send_button_p), GTK_RELIEF_NORMAL);
-  }
-    
-  else {
-    gtk_widget_set_sensitive(instance_p->view_button_p, false);
-    gtk_widget_set_sensitive(instance_p->remove_button_p, false);
-    gtk_widget_set_sensitive(instance_p->send_button_p, false);
-    gtk_button_set_relief(GTK_BUTTON(instance_p->view_button_p), GTK_RELIEF_NONE);
-    gtk_button_set_relief(GTK_BUTTON(instance_p->remove_button_p), GTK_RELIEF_NONE);
-    gtk_button_set_relief(GTK_BUTTON(instance_p->send_button_p), GTK_RELIEF_NONE);
-  }
-}
-
-gboolean SocketListDialogCB::tree_view_mouse_click(GtkWidget*,
-						   GdkEventButton* event_p, void* data) {
-  
-  if (event_p->type == GDK_2BUTTON_PRESS) {
-    SocketListDialog* instance_p = static_cast<SocketListDialog*>(data);
-    int x, y;
-    gtk_widget_get_pointer(GTK_WIDGET(instance_p->tree_view_p), &x, &y);
-    if (x >= 0 && y >= 0
-	&& gtk_tree_view_get_dest_row_at_pos(instance_p->tree_view_p,
-					     x, y, 0, 0)) {
-      instance_p->view_file();
-    }
-  }
-  return false;
-}
-
+} // namespace ModelColumns
 } // anonymous namespace
 
+class SocketListDialog::CB {
+public:
+  static void button_clicked(GtkWidget* widget_p,
+			     void* data) {
+    SocketListDialog* instance_p = static_cast<SocketListDialog*>(data);
+
+    if (widget_p == instance_p->send_button_p) {
+      instance_p->send_fax_impl();
+    }
+    else if (widget_p == instance_p->view_button_p) {
+      instance_p->view_file();
+    }
+    else if (widget_p == instance_p->remove_button_p) {
+      instance_p->remove_file_prompt();
+    }
+    else if (widget_p == instance_p->close_button_p) {
+      instance_p->close();
+    }
+    else {
+      write_error("Callback error in SocketListDialog::CB::button_clicked()\n");
+      instance_p->close();
+    }
+  }
+
+  static void set_buttons(void* data) {
+    SocketListDialog* instance_p = static_cast<SocketListDialog*>(data);
+
+    // see if anything is selected
+    GtkTreeSelection* selection_p = gtk_tree_view_get_selection(instance_p->tree_view_p);
+    if (gtk_tree_selection_get_selected(selection_p, 0, 0)) {
+      gtk_widget_set_sensitive(instance_p->view_button_p, true);
+      gtk_widget_set_sensitive(instance_p->remove_button_p, true);
+      gtk_widget_set_sensitive(instance_p->send_button_p, true);
+      gtk_button_set_relief(GTK_BUTTON(instance_p->view_button_p), GTK_RELIEF_NORMAL);
+      gtk_button_set_relief(GTK_BUTTON(instance_p->remove_button_p), GTK_RELIEF_NORMAL);
+      gtk_button_set_relief(GTK_BUTTON(instance_p->send_button_p), GTK_RELIEF_NORMAL);
+    }
+    
+    else {
+      gtk_widget_set_sensitive(instance_p->view_button_p, false);
+      gtk_widget_set_sensitive(instance_p->remove_button_p, false);
+      gtk_widget_set_sensitive(instance_p->send_button_p, false);
+      gtk_button_set_relief(GTK_BUTTON(instance_p->view_button_p), GTK_RELIEF_NONE);
+      gtk_button_set_relief(GTK_BUTTON(instance_p->remove_button_p), GTK_RELIEF_NONE);
+      gtk_button_set_relief(GTK_BUTTON(instance_p->send_button_p), GTK_RELIEF_NONE);
+    }
+  }
+
+  static gboolean tree_view_mouse_click(GdkEventButton* event_p,
+					void* data) {
+    if (event_p->type == GDK_2BUTTON_PRESS) {
+      SocketListDialog* instance_p = static_cast<SocketListDialog*>(data);
+      int x, y;
+      gtk_widget_get_pointer(GTK_WIDGET(instance_p->tree_view_p), &x, &y);
+      if (x >= 0 && y >= 0
+	  && gtk_tree_view_get_dest_row_at_pos(instance_p->tree_view_p,
+					       x, y, 0, 0)) {
+	instance_p->view_file();
+      }
+    }
+    return false;
+  }
+};
+
+extern "C" {
+  static void button_clicked_cb(GtkWidget* widget_p,
+				void* data) {
+    SocketListDialog::CB::button_clicked(widget_p, data);
+  }
+
+  static void set_buttons_cb(GtkTreeSelection*,
+			     void* data) {
+    SocketListDialog::CB::set_buttons(data);
+  }
+
+  static gboolean tree_view_mouse_click_cb(GtkWidget*,
+					   GdkEventButton* event_p,
+					   void* data) {
+    return SocketListDialog::CB::tree_view_mouse_click(event_p, data);
+  }
+
+} // extern "C"
 
 SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
 		                             SharedPtr<Thread::Mutex::Lock> > filenames_pair,
@@ -217,23 +236,23 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
   gtk_box_pack_start(GTK_BOX(socket_list_box_p), GTK_WIDGET(table_p), true, true, 0);
 
   g_signal_connect(G_OBJECT(send_button_p), "clicked",
-		   G_CALLBACK(SocketListDialogCB::socket_list_button_clicked), this);
+		   G_CALLBACK(button_clicked_cb), this);
   g_signal_connect(G_OBJECT(view_button_p), "clicked",
-		   G_CALLBACK(SocketListDialogCB::socket_list_button_clicked), this);
+		   G_CALLBACK(button_clicked_cb), this);
   g_signal_connect(G_OBJECT(remove_button_p), "clicked",
-		   G_CALLBACK(SocketListDialogCB::socket_list_button_clicked), this);
+		   G_CALLBACK(button_clicked_cb), this);
   g_signal_connect(G_OBJECT(close_button_p), "clicked",
-		   G_CALLBACK(SocketListDialogCB::socket_list_button_clicked), this);
+		   G_CALLBACK(button_clicked_cb), this);
 
   // now connect up the signal which indicates a selection has been made
   selection_p = gtk_tree_view_get_selection(tree_view_p);
   g_signal_connect(G_OBJECT(selection_p), "changed",
-		   G_CALLBACK(SocketListDialogCB::socket_list_set_buttons), this);
+		   G_CALLBACK(set_buttons_cb), this);
 
   // capture a double click on the tree view for viewing a fax for sending
   gtk_widget_add_events(GTK_WIDGET(tree_view_p), GDK_BUTTON_PRESS_MASK);
   g_signal_connect(G_OBJECT(tree_view_p), "button_press_event",
-		   G_CALLBACK(SocketListDialogCB::tree_view_mouse_click), this);
+		   G_CALLBACK(tree_view_mouse_click_cb), this);
 
 #if GTK_CHECK_VERSION(2,20,0)
   gtk_widget_set_can_default(close_button_p, true);
