@@ -1,4 +1,4 @@
-/* Copyright (C) 2008 Chris Vine
+/* Copyright (C) 2008 to 2010 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -28,8 +28,9 @@ License as follows:
 
 #include <unistd.h>
 
-#include <cstdlib>
-#include <cairo.h>
+#include <stdlib.h>
+#include <cstring>
+
 #include <tiffio.h>
 
 #include <glib.h>
@@ -80,71 +81,69 @@ inline uint32 cr_get_stride(uint32 width) {
 GobjHandle<GtkPrintSettings> MonoTiffPrintManager::print_settings_h;
 GobjHandle<GtkPageSetup> MonoTiffPrintManager::page_setup_h;
 
-namespace { // callbacks for internal use only
-
-void MonoTiffPrintManagerCB::mono_tiff_print_begin_print(GtkPrintOperation* print_operation_p,
-							 GtkPrintContext* context_p,
-							 void* data) {
-  static_cast<MonoTiffPrintManager*>(data)->begin_print_impl(print_operation_p, context_p);
-}
-
-void MonoTiffPrintManagerCB::mono_tiff_print_draw_page(GtkPrintOperation* print_operation_p,
-						       GtkPrintContext* context_p,
-						       gint page_nr,
-						       void* data) {
-  static_cast<MonoTiffPrintManager*>(data)->draw_page_impl(print_operation_p, context_p, page_nr);
-}
-
-void MonoTiffPrintManagerCB::mono_tiff_print_done(GtkPrintOperation* print_operation_p,
-						  GtkPrintOperationResult result,
-						  void* data) {
-
-  MonoTiffPrintManager* instance_p = static_cast<MonoTiffPrintManager*>(data);
-  if (result == GTK_PRINT_OPERATION_RESULT_ERROR) {
-    write_error("Print error in MonoTiffPrintManagerCB::mono_tiff_print_done()\n");
-    GError* error_p;
-    gtk_print_operation_get_error(print_operation_p, &error_p);
-    GerrorScopedHandle handle_h(error_p);
-    write_error(error_p->message);
-    write_error("\n");
+class MonoTiffPrintManager::CB {
+public:
+  static void begin_print(GtkPrintOperation* print_operation_p,
+			  GtkPrintContext* context_p,
+			  void* data) {
+    static_cast<MonoTiffPrintManager*>(data)->begin_print_impl(print_operation_p, context_p);
   }
-  else if (result == GTK_PRINT_OPERATION_RESULT_APPLY) {
-    // save the print settings
-    MonoTiffPrintManager::print_settings_h =
-      GobjHandle<GtkPrintSettings>(gtk_print_operation_get_print_settings(print_operation_p));
-    // and take ownership of the GtkPrintSettings object
-    g_object_ref(G_OBJECT(MonoTiffPrintManager::print_settings_h.get()));
-  }
-  else if (result == GTK_PRINT_OPERATION_RESULT_CANCEL
-	   && instance_p->parent_p
-	   && !instance_p->cancelled_when_drawing) {
-    // the parent will not be set sensitive if MonoTiffPrintManager::begin_print_impl()
-    // has not been entered because the print job was cancelled from the print dialog
-    gtk_widget_set_sensitive(GTK_WIDGET(instance_p->parent_p), true);
-  }
-  { // scope block for mutex lock
-    Thread::Mutex::Lock lock(instance_p->mutex);
-    instance_p->ready = true;
-  }
-  instance_p->unref();
-}
 
-void MonoTiffPrintManagerCB::mono_tiff_print_page_setup_done(GtkPageSetup* page_setup_p, void* data) {
-
-  if (page_setup_p) {
-    // The documentation is unclear whether we get a new GtkPageSetup
-    // object back if an existing GtkPageSetup object was passed to
-    // gtk_print_run_page_setup_dialog_async().  However this is safe
-    // even if we get the same object back again - see GobjHandle
-    // implementation
-    MonoTiffPrintManager::page_setup_h = GobjHandle<GtkPageSetup>(page_setup_p);
+  static void draw_page(GtkPrintOperation* print_operation_p,
+			GtkPrintContext* context_p,
+			gint page_nr,
+			void* data) {
+    static_cast<MonoTiffPrintManager*>(data)->draw_page_impl(print_operation_p, context_p, page_nr);
   }
-  if (data) gtk_widget_set_sensitive(GTK_WIDGET(data), true);
-}
 
-void MonoTiffPrintManagerCB::mono_tiff_print_request_page_setup(GtkPrintOperation*, GtkPrintContext*,
-								gint, GtkPageSetup* page_setup_p,
-								void*) {
+  static void done(GtkPrintOperation* print_operation_p,
+		   GtkPrintOperationResult result,
+		   void* data) {
+
+    MonoTiffPrintManager* instance_p = static_cast<MonoTiffPrintManager*>(data);
+    if (result == GTK_PRINT_OPERATION_RESULT_ERROR) {
+      write_error("Print error in MonoTiffPrintManagerCB::mono_tiff_print_done()\n");
+      GError* error_p;
+      gtk_print_operation_get_error(print_operation_p, &error_p);
+      GerrorScopedHandle handle_h(error_p);
+      write_error(error_p->message);
+      write_error("\n");
+    }
+    else if (result == GTK_PRINT_OPERATION_RESULT_APPLY) {
+      // save the print settings
+      MonoTiffPrintManager::print_settings_h =
+	GobjHandle<GtkPrintSettings>(gtk_print_operation_get_print_settings(print_operation_p));
+      // and take ownership of the GtkPrintSettings object
+      g_object_ref(G_OBJECT(MonoTiffPrintManager::print_settings_h.get()));
+    }
+    else if (result == GTK_PRINT_OPERATION_RESULT_CANCEL
+	     && instance_p->parent_p
+	     && !instance_p->cancelled_when_drawing) {
+      // the parent will not be set sensitive if MonoTiffPrintManager::begin_print_impl()
+      // has not been entered because the print job was cancelled from the print dialog
+      gtk_widget_set_sensitive(GTK_WIDGET(instance_p->parent_p), true);
+    }
+    { // scope block for mutex lock
+      Thread::Mutex::Lock lock(instance_p->mutex);
+      instance_p->ready = true;
+    }
+    instance_p->unref();
+  }
+
+  static void page_setup_done(GtkPageSetup* page_setup_p, void* data) {
+
+    if (page_setup_p) {
+      // The documentation is unclear whether we get a new GtkPageSetup
+      // object back if an existing GtkPageSetup object was passed to
+      // gtk_print_run_page_setup_dialog_async().  However this is safe
+      // even if we get the same object back again - see GobjHandle
+      // implementation
+      MonoTiffPrintManager::page_setup_h = GobjHandle<GtkPageSetup>(page_setup_p);
+    }
+    if (data) gtk_widget_set_sensitive(GTK_WIDGET(data), true);
+  }
+
+  static void request_page_setup(GtkPageSetup* page_setup_p) {
     // we want to remove the right and bottom margins, as these will
     // be established by the fax as sent.  If the recipient wants to
     // scale them to fit different printer margins, the user should
@@ -153,9 +152,43 @@ void MonoTiffPrintManagerCB::mono_tiff_print_request_page_setup(GtkPrintOperatio
     // the draw_page_impl() method will automatically scale for these.
     gtk_page_setup_set_bottom_margin(page_setup_p, 0.0, GTK_UNIT_MM);
     gtk_page_setup_set_right_margin(page_setup_p, 0.0, GTK_UNIT_MM);
-}
+  }
+};
+
+// the GObject callback functions with both C linkage
+// specification and internal linkage
+extern "C" {
+
+  static void mtp_begin_print(GtkPrintOperation* print_operation_p,
+			      GtkPrintContext* context_p,
+			      void* data) {
+    MonoTiffPrintManager::CB::begin_print(print_operation_p, context_p, data);
+  }
+
+  static void mtp_draw_page(GtkPrintOperation* print_operation_p,
+			    GtkPrintContext* context_p,
+			    gint page_nr,
+			    void* data) {
+    MonoTiffPrintManager::CB::draw_page(print_operation_p, context_p, page_nr, data);
+  }
+
+  static void mtp_done(GtkPrintOperation* print_operation_p,
+		       GtkPrintOperationResult result,
+		       void* data) {
+    MonoTiffPrintManager::CB::done(print_operation_p, result, data);
+  }
+
+  static void mtp_page_setup_done(GtkPageSetup* page_setup_p, void* data) {
+    MonoTiffPrintManager::CB::page_setup_done(page_setup_p, data);
+  }
+
+  static void mtp_request_page_setup(GtkPrintOperation*, GtkPrintContext*,
+				     gint, GtkPageSetup* page_setup_p, void*) {
+    MonoTiffPrintManager::CB::request_page_setup(page_setup_p);
+  }
+
+} // extern "C"
 
-} // anonymous namespace
 
 MonoTiffPrintManager::~MonoTiffPrintManager(void) {
 
@@ -168,6 +201,7 @@ IntrusivePtr<MonoTiffPrintManager> MonoTiffPrintManager::create_manager(GtkWindo
 
   MonoTiffPrintManager* instance_p = new MonoTiffPrintManager;
   instance_p->print_notifier.connect(Callback::make(*instance_p, &MonoTiffPrintManager::print_files));
+  std::memset(&(instance_p->destroy_key), 0, sizeof(cairo_user_data_key_t));
   instance_p->parent_p = parent;
   instance_p->cancelled_when_drawing = false;
 
@@ -185,7 +219,7 @@ void MonoTiffPrintManager::page_setup(GtkWindow* parent) {
 
   if (parent) gtk_widget_set_sensitive(GTK_WIDGET(parent), false);
   gtk_print_run_page_setup_dialog_async(parent, page_setup_h, print_settings_h, 
-					MonoTiffPrintManagerCB::mono_tiff_print_page_setup_done,
+					mtp_page_setup_done,
 					parent);
 }
 
@@ -274,14 +308,14 @@ void MonoTiffPrintManager::print_files(void) {
   }
 
   g_signal_connect(G_OBJECT(print_operation_h.get()), "begin_print",
-		   G_CALLBACK(MonoTiffPrintManagerCB::mono_tiff_print_begin_print), this);
+		   G_CALLBACK(mtp_begin_print), this);
   g_signal_connect(G_OBJECT(print_operation_h.get()), "draw_page",
-		   G_CALLBACK(MonoTiffPrintManagerCB::mono_tiff_print_draw_page), this);
+		   G_CALLBACK(mtp_draw_page), this);
   g_signal_connect(G_OBJECT(print_operation_h.get()), "request_page_setup",
-		   G_CALLBACK(MonoTiffPrintManagerCB::mono_tiff_print_request_page_setup), 0);
+		   G_CALLBACK(mtp_request_page_setup), 0);
 #ifdef MONO_TIFF_PRINT_MANAGER_ALLOW_ASYNC
   g_signal_connect(G_OBJECT(print_operation_h.get()), "done",
-		   G_CALLBACK(MonoTiffPrintManagerCB::mono_tiff_print_done), this);
+		   G_CALLBACK(mtp_done), this);
   gtk_print_operation_set_allow_async(print_operation_h, true);
   // regain ownership of ourselves (the print system will do the final
   // unreference in the MonoTiffPrintManagerCB::mono_tiff_print_done() callback)
@@ -421,35 +455,39 @@ void MonoTiffPrintManager::draw_page_impl(GtkPrintOperation* print_operation_p,
   }
 
   // provide a buffer for the image raster
-  // functor class CFree is defined in shared_handle.h
-  ScopedHandle<unsigned char*, CFree> buf_h(static_cast<unsigned char*>(std::calloc(cr_stride * height, 1)));
-  if (!buf_h.get()) {
+  unsigned char* buf = static_cast<unsigned char*>(calloc(cr_stride * height, 1));
+  if (!buf) {
     write_error("Memory allocation error in MonoTiffPrintManager::draw_page_impl()\n");
-    std::exit(MEM_ERROR);
+    exit(MEM_ERROR);
   }
-  unsigned char* buf = buf_h.get();
 
   // fill up the image raster buffer
   uint32 row;
-  for (row = 0; row < height; ++row, buf += cr_stride) {
-    TIFFReadScanline(tif_h, buf, row);
+  unsigned char* tmp = buf;
+  for (row = 0; row < height; ++row, tmp += cr_stride) {
+    TIFFReadScanline(tif_h, tmp, row);
   }
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
-  TIFFReverseBits(buf_h.get(), cr_stride * height);
+  TIFFReverseBits(buf, cr_stride * height);
 #endif
 
   // now form the image surface
-  CairoSurfaceScopedHandle surface_h(cairo_image_surface_create_for_data(buf_h, CAIRO_FORMAT_A1,
+  CairoSurfaceScopedHandle surface_h(cairo_image_surface_create_for_data(buf, CAIRO_FORMAT_A1,
 									 width, height,
 									 cr_stride));
 
   if (cairo_surface_status(surface_h) != CAIRO_STATUS_SUCCESS) {
     write_error("Cairo surface error in MonoTiffPrintManager::draw_page_impl()\n");
     cancelled_when_drawing = true;
-    gtk_print_operation_cancel(print_operation_p);    
+    gtk_print_operation_cancel(print_operation_p);
+    free(buf);
   }
   else {
+    // the gtk+ print system keeps a reference to the surface after
+    // this method returns so set up a callback to free buf
+    cairo_surface_set_user_data(surface_h, &destroy_key, buf, (cairo_destroy_func_t)free);
+
     // get the cairo context and scale it so that the pixel size of
     // the image pattern matches the pixel size of the print context
     double width_ratio = gtk_print_context_get_width(context_p)/width;
